@@ -22,14 +22,11 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gtk/gtkmenushell.h>
+#include <gtk/gtkstatusbar.h>
 #include "bookmarks.h"
 #include "bookmarks-util.h"
 #include "bookmarks-location-source.h"
 
-#define GB_GTK_MENU_NAME_MAX_LENGTH 40 /** Max length of a bookmark menu item */
-#define GB_GTK_MENU_TIP_MAX_LENGTH 30 /** Max length of a bookmark url tooltip */
-
-
 /* object forward declarations */
 
 typedef struct _GbGtkMenu GbGtkMenu;
@@ -70,7 +67,9 @@ struct _GbGtkMenu
 
 GType		gb_gtk_menu_get_type			(void);
 GbGtkMenu *	gb_gtk_menu_new				(GbFolder *root, GtkMenuShell *ms);
+GbGtkMenu *	gb_gtk_menu_new_context_only		(GbBookmarkSet *set, GtkMenuShell *ms);
 void		gb_gtk_menu_set_location_source		(GbGtkMenu *gm, GbLocationSource *src);
 void		gb_gtk_menu_fill_children_submenus	(GbGtkMenu *gm);
+void		gb_gtk_menu_set_statusbar 		(GbGtkMenu *gm, GtkStatusbar *sb);
 
 #endif
