@@ -1,3 +1,4 @@
+/* -*- mode: c; c-style: k&r; c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2002  Ricardo Fernndez Pascual
  *
@@ -23,11 +24,11 @@
 #include "bookmarks-smart-site-tb-widget.h"
 #include "galeon-marshal.h"
 #include "pixbuf-cache.h"
-#include "gul-gobject-misc.h"
 #include "gul-gui.h"
 #include "bookmarks-widgets-private.h"
 #include "bookmarks-single-editor.h"
 #include "bookmarks-dnd.h"
+#include "bookmarks-context-menu.h"
 #include "galeon-dnd.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
@@ -48,6 +49,10 @@
 /**
  * Private data
  */
+#define GB_SMART_SITE_TB_WIDGET_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GB_TYPE_SMART_SITE_TB_WIDGET, GbSmartSiteTbWidgetPrivate))
+
+
 struct _GbSmartSiteTbWidgetPrivate {
 	GtkWidget *mainwid;
 	GtkWidget *main_button;
@@ -78,8 +83,6 @@ const GtkTargetEntry text_drop_types[] =
 /**
  * Private functions, only availble from this file
  */
-static void		gb_smart_site_tb_widget_class_init	(GbSmartSiteTbWidgetClass *klass);
-static void		gb_smart_site_tb_widget_init		(GbSmartSiteTbWidget *w);
 static void		gb_smart_site_tb_widget_finalize_impl	(GObject *o);
 static void		gb_smart_site_tb_widget_rebuild_impl	(GbTbWidget *w);
 static void		gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w);
@@ -109,35 +112,37 @@ static void		gb_smart_site_tb_widget_drag_data_received_cb (GtkWidget *widget,
 								       guint info,
 								       guint t,
 								       GbSmartSiteTbWidget *w);
+static gboolean 	gb_smart_site_tb_widget_entries_empty	(GbSmartSiteTbWidget *w);
+static void		gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w, 
+								    GbBookmarkActivatedFlags flags);
+
 
 
-static gpointer gb_tb_widget_class;
 
 /**
  * GbSmartSiteTbWidget object
  */
 
-MAKE_GET_TYPE (gb_smart_site_tb_widget, "GbSmartSiteTbWidget", GbSmartSiteTbWidget, 
-	       gb_smart_site_tb_widget_class_init, 
-	       gb_smart_site_tb_widget_init, GB_TYPE_TB_WIDGET);
+G_DEFINE_TYPE (GbSmartSiteTbWidget, gb_smart_site_tb_widget, GB_TYPE_TB_WIDGET);
 
 static void
 gb_smart_site_tb_widget_class_init (GbSmartSiteTbWidgetClass *klass)
 {
 	G_OBJECT_CLASS (klass)->finalize = gb_smart_site_tb_widget_finalize_impl;
 	GB_TB_WIDGET_CLASS (klass)->rebuild = gb_smart_site_tb_widget_rebuild_impl;
-	gb_tb_widget_class = g_type_class_peek_parent (klass);
 
 	eel_gconf_notification_add 
 		(CONF_BOOKMARKS_HISTORY, gb_smart_site_tb_widget_conf_changed_cb, NULL);
 	eel_gconf_notification_add 
 		(CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS, gb_smart_site_tb_widget_conf_changed_cb, NULL);
+
+	g_type_class_add_private (klass, sizeof (GbSmartSiteTbWidgetPrivate));
 }
 
 static void 
 gb_smart_site_tb_widget_init (GbSmartSiteTbWidget *w)
 {
-	GbSmartSiteTbWidgetPrivate *p = g_new0 (GbSmartSiteTbWidgetPrivate, 1);
+	GbSmartSiteTbWidgetPrivate *p = GB_SMART_SITE_TB_WIDGET_GET_PRIVATE (w);
 	w->priv = p;
 
 	all_instances = g_slist_prepend (all_instances, w);
@@ -155,9 +160,8 @@ gb_smart_site_tb_widget_finalize_impl (GObject *o)
 	g_free (p->combos);
 	g_free (p->entries_sizes);
 
-	g_free (p);
 	
-	G_OBJECT_CLASS (gb_tb_widget_class)->finalize (o);
+	G_OBJECT_CLASS (gb_smart_site_tb_widget_parent_class)->finalize (o);
 }
 
 GbSmartSiteTbWidget *
@@ -298,6 +302,75 @@ gb_smart_site_tb_widget_build_smart_box (GbSmartSiteTbWidget *w)
 			  G_CALLBACK (gb_smart_site_tb_widget_entry_width_changed_cb), w);
 }
 
+static void
+gb_smart_site_context_menu_bookmark_activated_cb (GbContextMenu *gm, 
+						  GbBookmarkEventActivated *ev,
+						  GbSmartSiteTbWidget *w)
+{
+	if (gb_smart_site_tb_widget_entries_empty (w))
+	{
+		gb_smart_site_tb_widget_do_the_smart_thing (w, ev->flags);
+	}
+	else
+	{
+		g_signal_emit_by_name (w, "bookmark-activated", ev);
+	}
+}
+
+static void
+gb_smart_site_tb_widget_popup_menu (GbSmartSiteTbWidget *w, GdkEventButton *event)
+{
+	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
+	GbLocationSource *location_source = gb_tb_widget_get_location_source (GB_TB_WIDGET (w));
+	GbContextMenu *cm = gb_context_menu_new ();
+	gb_context_menu_set_bookmark (cm, b);
+	gb_context_menu_set_location_source (cm, location_source);
+	
+	g_signal_connect (cm, "bookmark-activated", 
+			  G_CALLBACK (gb_smart_site_context_menu_bookmark_activated_cb), 
+			  w);
+	gb_context_menu_popup (cm, event);
+	
+	g_object_unref (cm);
+}
+
+static gboolean
+gb_smart_site_tb_widget_popup_menu_cb (GtkWidget *b, GbSmartSiteTbWidget *w)
+{
+	gb_smart_site_tb_widget_popup_menu (w, NULL);
+	return TRUE;
+}
+
+static gboolean
+gb_smart_site_tb_widget_button_press_cb (GtkWidget *wid, 
+					 GdkEventButton *event, 
+					 GbSmartSiteTbWidget *w)
+{
+	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
+	g_return_val_if_fail (GB_IS_SMART_SITE (b), FALSE);
+	
+	if (b && event->button == 2)
+	{
+		if (gb_smart_site_tb_widget_entries_empty (w))
+		{
+			gb_smart_site_tb_widget_do_the_smart_thing (w, GB_BAF_NEW_TAB_OR_WINDOW);
+		}
+		else
+		{
+			gb_activated_event (w, b, GB_SITE (b)->url, 
+					    GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
+		}
+		return TRUE;
+	}
+	else if (event->button == 3)
+	{
+		gb_smart_site_tb_widget_popup_menu (w, event);
+		return TRUE;
+	}
+	return FALSE;
+
+}
+
 static void
 gb_smart_site_tb_widget_rebuild_impl (GbTbWidget *w)
 {
@@ -329,7 +402,11 @@ gb_smart_site_tb_widget_rebuild_impl (GbTbWidget *w)
 	gtk_widget_show (GTK_WIDGET (p->main_button));
 	gb_widgets_fill_tb_item (GTK_CONTAINER (p->main_button), GB_BOOKMARK (site), GB_SITE (site)->url);
 
-	gb_tb_widget_setup_context_menu (w, p->main_button);
+	g_signal_connect (p->main_button, "popup_menu", 
+			  G_CALLBACK (gb_smart_site_tb_widget_popup_menu_cb), w);
+	g_signal_connect (p->main_button, "button_press_event",
+			  G_CALLBACK (gb_smart_site_tb_widget_button_press_cb), w);
+
 	gb_bookmark_dnd_drag_source_set (GTK_BIN (p->main_button)->child, GB_BOOKMARK (site));
 
 	if (gb_smart_site_get_num_fields (site) == 1 )
@@ -408,7 +485,7 @@ gb_smart_site_tb_widget_visibility_changed_cb (GbSmartSite *ss, GbSmartSiteTbWid
 
 static void
 gb_smart_site_tb_widget_do_the_smart_thing (GbSmartSiteTbWidget *w, 
-					     GbBookmarkActivatedFlags flags)
+					    GbBookmarkActivatedFlags flags)
 {
 	GbSmartSiteTbWidgetPrivate *p = w->priv;
 	gchar *url;
@@ -515,29 +592,6 @@ gb_smart_site_tb_widget_entry_width_changed_cb (GbSmartSite *b, gint index, guin
 	gtk_widget_set_size_request (p->entries[index], p->entries_sizes[index], -1);
 }
 
-
-static gboolean
-gb_smart_site_tb_widget_button_press_cb (GtkWidget *wid, 
-					 GdkEventButton *event, 
-					 GbSmartSiteTbWidget *w)
-{
-	GbBookmark *b = gb_tb_widget_get_bookmark (GB_TB_WIDGET (w));
-	if (b && event->button == 2)
-	{
-		if (gb_smart_site_tb_widget_entries_empty (w))
-		{
-			gb_smart_site_tb_widget_do_the_smart_thing (w, GB_BAF_NEW_TAB_OR_WINDOW);
-		}
-		else
-		{
-			gb_activated_event (w, b, GB_SITE (b)->url, 
-					    GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
-		}
-		return TRUE;
-	}
-	return FALSE;
-}
-
 static void
 gb_smart_site_tb_widget_history_changed_cb (GbSmartSite *ss, GbSmartSiteTbWidget *w)
 {
