@@ -1,3 +1,4 @@
+/* -*- mode: c; c-style: k&r; c-basic-offset: 8 -*- */
 /* 
  *  Copyright (C) 2002 Ricardo Fernndez Pascual
  *
@@ -37,7 +38,6 @@
 #include <libgnome/gnome-util.h>
 
 #include "xbel.h"
-#include "gul-gobject-misc.h"
 
 #define XBEL_GALEON_OWNER "http://galeon.sourceforge.net/"
 
@@ -49,8 +49,6 @@ typedef struct _GbXBELMetadata GbXBELMetadata;
 /**
  * Private functions, only availble from this file
  */
-static void		gb_xbel_class_init		(GbXBELClass *klass);
-static void		gb_xbel_init			(GbXBEL *io);
 static GbBookmarkSet *	gb_xbel_load_from_file		(GbIO *io, const gchar *filename);
 static GbBookmarkSet *	gb_xbel_load_from_string	(GbIO *io, const gchar *data);
 static gboolean		gb_xbel_save_to_file		(GbIO *io, GbBookmarkSet *set,
@@ -113,7 +111,14 @@ struct _GbXBELMetadata
 	gboolean toolbar;
 	gboolean context;
 	gboolean default_f;
-	gboolean autobm;
+	gboolean autofolder;
+	gboolean vfolder;
+	gchar *vfolder_search_text;
+	gchar *vfolder_search_options;
+	gchar *vfolder_search_recently_created;
+	gchar *vfolder_search_recently_visited;
+	gchar *autobm_search_text;
+	gchar *autobm_search_options;
 	GTime t_add;
 	GTime t_vis;
 	GTime t_mod;
@@ -128,8 +133,7 @@ struct _GbXBELMetadata
  * XBEL object
  */
 
-MAKE_GET_TYPE (gb_xbel, "GbXBEL", GbXBEL, gb_xbel_class_init, 
-	       gb_xbel_init, GB_TYPE_IO);
+G_DEFINE_TYPE (GbXBEL, gb_xbel, GB_TYPE_IO);
 
 static void
 gb_xbel_class_init (GbXBELClass *klass)
@@ -296,7 +300,13 @@ xbel_read (xmlDocPtr doc)
 	set = gb_bookmark_set_new ();
 	gb_bookmark_set_set_xbel_doc (set, doc);
 	root = xbel_read_node (set, item);
-	
+	if (!root)
+	{
+		g_object_unref (set);
+		return NULL;
+	}
+
+
 	if (!set->root && GB_IS_FOLDER (root))
 	{
 		gb_bookmark_set_set_root (set, GB_FOLDER (root));
@@ -322,7 +332,7 @@ xbel_read_node (GbBookmarkSet *set, xmlNodePtr node)
 		g_warning ("xml node with NULL name!");
 	} 
 	else if (!strcmp (node->name, "folder")
-	    || !strcmp (node->name, "xbel")) 
+		 || !strcmp (node->name, "xbel")) 
 	{
 		GbFolder *folder;
 		gchar *title = get_child_value_string (node, "title");
@@ -351,9 +361,48 @@ xbel_read_node (GbBookmarkSet *set, xmlNodePtr node)
 			else
 				title = g_strdup (_("Untitled folder"));
 		}
-		if (md->autobm)
+		if (md->vfolder)
 		{
-			folder = (GbFolder *) gb_auto_folder_new (set, title);
+			GbVFolder *vf = gb_v_folder_new (set, title);
+			folder = (GbFolder *) vf;
+			if (md->vfolder_search_text)
+			{
+				gb_v_folder_set_search_text (vf, md->vfolder_search_text);
+			}
+			if (md->vfolder_search_options)
+			{
+				gb_v_folder_set_search_options (vf, md->vfolder_search_options);
+			}
+			if (md->vfolder_search_recently_visited)
+			{
+				int days = atoi (md->vfolder_search_recently_visited);
+				gb_v_folder_set_rencently_visited (vf, TRUE, days);
+			}
+			else
+			{
+				gb_v_folder_set_rencently_visited (vf, FALSE, 0);
+			}
+			if (md->vfolder_search_recently_created)
+			{
+				int days = atoi (md->vfolder_search_recently_created);
+				gb_v_folder_set_rencently_created (vf, TRUE, days);
+			}
+			else
+			{
+				gb_v_folder_set_rencently_created (vf, FALSE, 0);
+			}
+		}
+		else if (md->autofolder)
+		{
+			folder = GB_FOLDER (gb_auto_folder_new (set, title));
+			if (md->autobm_search_text)
+			{
+				gb_auto_folder_set_search_text (GB_AUTO_FOLDER (folder), md->autobm_search_text);
+			}
+			if (md->autobm_search_options)
+			{
+				gb_auto_folder_set_options (GB_AUTO_FOLDER (folder), md->autobm_search_options);
+			}
 		}
 		else
 		{
@@ -487,7 +536,7 @@ xbel_read_node (GbBookmarkSet *set, xmlNodePtr node)
 static GbXBELMetadata *
 xbel_read_metadata (xmlNodePtr node)
 {
-        xmlNodePtr info_node = get_child (node, "info");
+	xmlNodePtr info_node = get_child (node, "info");
         if (info_node) 
 	{
                 /* Iterate over the metadata children */
@@ -507,6 +556,7 @@ xbel_read_metadata (xmlNodePtr node)
 			}
 		}
 	}
+
 	return gb_xbel_metadata_new ();
 }
 
@@ -528,7 +578,14 @@ xbel_read_metadata_node (xmlNodePtr node)
 	md->toolbar = get_child_value_boolean (node, "create_toolbar");
 	md->context = get_child_value_boolean (node, "create_context");
 	md->default_f = get_child_value_boolean (node, "default_folder");
-	md->autobm = get_child_value_boolean (node, "autobookmarks");
+	md->autofolder = get_child_value_boolean (node, "autofolder");
+	md->vfolder = get_child_value_boolean (node, "vfolder");
+	md->vfolder_search_text = get_child_value_string (node, "vfolder_search_text");
+	md->vfolder_search_options = get_child_value_string (node, "vfolder_search_options");
+	md->vfolder_search_recently_visited = get_child_value_string (node, "vfolder_search_recently_visited");
+	md->vfolder_search_recently_created = get_child_value_string (node, "vfolder_search_recently_created");
+	md->autobm_search_text = get_child_value_string (node, "autobm_search_text");
+	md->autobm_search_options = get_child_value_string (node, "autobm_options");
 	md->t_add = get_child_value_long (node, "time_added");
 	md->t_vis = get_child_value_long (node, "time_visited");
 	md->t_mod = get_child_value_long (node, "time_modified");
@@ -613,6 +670,12 @@ gb_xbel_metadata_free (GbXBELMetadata *md)
 	g_free (md->nick);
 	g_free (md->smarturl);
 	g_free (md->pixmap);
+	g_free (md->vfolder_search_text);
+	g_free (md->vfolder_search_options);
+	g_free (md->vfolder_search_recently_visited);
+	g_free (md->vfolder_search_recently_created);
+	g_free (md->autobm_search_text);
+	g_free (md->autobm_search_options);
 	g_free (md);
 }
 
@@ -687,7 +750,8 @@ xbel_save_root (GbBookmarkSet *set)
 		xml_node_update_text_child (newNode, NULL, "title", ((GbBookmark *) b)->name);
 		md = xbel_save_common_data (newNode, (GbBookmark *) b);
 		
-		md->autobm = GB_IS_AUTO_FOLDER (b);
+		md->autofolder = GB_IS_AUTO_FOLDER (b);
+		md->vfolder = GB_IS_V_FOLDER (b);
 		md->toolbar = b->create_toolbar;
 		md->default_f = b == set->default_folder;
 		/* TODO: md->tb_style =  */
@@ -695,8 +759,13 @@ xbel_save_root (GbBookmarkSet *set)
 		xbel_save_galeon_metadata (newNode, md);
 		gb_xbel_metadata_free (md);
 		
-		for (child = b->child; child != NULL; child = child->next)
-			xbel_save_node (newNode, set, child);
+		if (!gb_folder_is_autogenerated (b))
+		{
+			for (child = b->child; child != NULL; child = child->next)
+			{
+				xbel_save_node (newNode, set, child);
+			}
+		}
 	}
 	return doc;
 }
@@ -877,16 +946,39 @@ xbel_save_folder (xmlNodePtr parent_node, GbBookmarkSet *set, GbFolder *b)
 
 	md = xbel_save_common_data (newNode, GB_BOOKMARK (b));
 	
-	md->autobm = GB_IS_AUTO_FOLDER (b);
 	md->toolbar = b->create_toolbar;
 	md->default_f = b == set->default_folder;
 	/* TODO: md->tb_style =  */
+
+	md->autofolder = GB_IS_AUTO_FOLDER (b);
 	
+	md->vfolder = GB_IS_V_FOLDER (b);
+	if (md->vfolder)
+	{
+		GbVFolder *vf = GB_V_FOLDER (b);
+		int days;
+		md->vfolder_search_text = g_strdup (gb_v_folder_get_search_text (vf));
+		md->vfolder_search_options = gb_v_folder_get_search_options (vf);
+		if (gb_v_folder_get_rencently_visited (vf, &days))
+		{
+			md->vfolder_search_recently_visited = g_strdup_printf ("%d", days);
+		}
+		if (gb_v_folder_get_rencently_created (vf, &days))
+		{
+			md->vfolder_search_recently_created = g_strdup_printf ("%d", days);
+		}
+	}
+	if (md->autofolder)
+	{
+		md->autobm_search_text = g_strdup (gb_auto_folder_get_search_text (GB_AUTO_FOLDER (b)));
+		md->autobm_search_options = gb_auto_folder_get_options (GB_AUTO_FOLDER (b));
+	}
+
 	xbel_save_galeon_metadata (newNode, md);
 	gb_xbel_metadata_free (md);
 	
         /* now iterate over the children of this folder */
-	if (!gb_bookmark_is_alias (b))
+	if (!gb_bookmark_is_alias (b) && !gb_folder_is_autogenerated (b))
 	{
 		GbBookmark *child;
 		for (child = b->child; child != NULL; child = child->next)
@@ -1111,12 +1203,14 @@ xbel_save_galeon_metadata (xmlNodePtr node, GbXBELMetadata *md)
 	    || md->t_vis
 	    || md->acc_key
 	    || md->acc_mod
-	    || md->autobm
+	    || md->vfolder
+	    || md->autofolder
 	    || (sstrlen (md->nick) > 0) 
 	    || (sstrlen (md->pixmap) > 0) 
 	    || (sstrlen (md->smarturl) > 0)
 	    || md->version
-	    || md->smart_site_history)
+	    || md->smart_site_history
+	    || md->vfolder)
 	{
 		xmlNodePtr metaNode;
 		GString *s;
@@ -1277,13 +1371,80 @@ xbel_save_galeon_metadata (xmlNodePtr node, GbXBELMetadata *md)
 			xml_node_remove_child (metaNode, "smart_site_history");
 		}
 
-		if (md->autobm)
+		if (md->autofolder)
 		{
-			xml_node_update_text_child (metaNode, NULL, "autobookmarks", "yes");
+			xml_node_update_text_child (metaNode, NULL, "autofolder", "yes");
+			if (md->autobm_search_text)
+			{
+				xml_node_update_text_child (metaNode, NULL, "autobm_search_text", md->autobm_search_text);
+			}
+			else
+			{
+				xml_node_remove_child (metaNode, "autobm_search_text");
+			}
+
+			if (md->autobm_search_options)
+			{
+				xml_node_update_text_child (metaNode, NULL, "autobm_options", md->autobm_search_options);
+			}
+			else
+			{
+				xml_node_remove_child (metaNode, "autobm_options");
+			}
 		}
 		else
 		{
-			xml_node_remove_child (metaNode, "autobookmarks");
+			xml_node_remove_child (metaNode, "autofolder");
+			xml_node_remove_child (metaNode, "autobm_options");
+			xml_node_remove_child (metaNode, "autobm_search_text");
+		}
+
+		if (md->vfolder)
+		{
+			xml_node_update_text_child (metaNode, NULL, "vfolder", "yes");
+			if (md->vfolder_search_text)
+			{
+				xml_node_update_text_child (metaNode, NULL, "vfolder_search_text", md->vfolder_search_text);
+			}
+			else
+			{
+				xml_node_remove_child (metaNode, "vfolder_search_text");
+			}
+
+			if (md->vfolder_search_options)
+			{
+				xml_node_update_text_child (metaNode, NULL, "vfolder_search_options", md->vfolder_search_options);
+			}
+			else
+			{
+				xml_node_remove_child (metaNode, "vfolder_search_options");
+			}
+
+			if (md->vfolder_search_recently_created)
+			{
+				xml_node_update_text_child (metaNode, NULL, "vfolder_search_recently_created", md->vfolder_search_recently_created);
+			}
+			else
+			{
+				xml_node_remove_child (metaNode, "vfolder_search_recently_created");
+			}
+
+			if (md->vfolder_search_recently_visited)
+			{
+				xml_node_update_text_child (metaNode, NULL, "vfolder_search_recently_visited", md->vfolder_search_recently_visited);
+			}
+			else
+			{
+				xml_node_remove_child (metaNode, "vfolder_search_recently_visited");
+			}
+		}
+		else
+		{
+			xml_node_remove_child (metaNode, "vfolder");
+			xml_node_remove_child (metaNode, "vfolder_search_text");
+			xml_node_remove_child (metaNode, "vfolder_search_options");
+			xml_node_remove_child (metaNode, "vfolder_search_recently_created");
+			xml_node_remove_child (metaNode, "vfolder_search_recently_visited");
 		}
 
 		if (md->version)
@@ -1301,6 +1462,7 @@ xbel_save_galeon_metadata (xmlNodePtr node, GbXBELMetadata *md)
 				break;
 			}
 		}
+
 		g_string_free (s, TRUE);
         }
 	else
