@@ -21,21 +21,9 @@
 #endif
 
 #include "galeon-embed.h"
+#include "galeon-embed-shell.h"
 #include "galeon-marshal.h"
-
-#if !defined (ENABLE_MOZILLA_EMBED) && !defined (ENABLE_GTKHTML_EMBED)
-#error "At least one embed needs to be enabled"
-#endif
-
-#ifdef ENABLE_MOZILLA_EMBED
-#include "mozilla-embed.h"
-#include "mozilla-embed-shell.h"
-#endif
-
-#ifdef ENABLE_GTKHTML_EMBED
-#include "gtkhtml-embed.h"
-#include "gtkhtml-embed-shell.h"
-#endif
+#include <string.h>
 
 enum
 {
@@ -57,17 +45,14 @@ enum
 	FAVICON,
 	POPUP_BLOCKED,
 	CONTEXTMENU,
+	NET_STOP,
+	NET_START,
 	LAST_SIGNAL
 };
 
 static void
 galeon_embed_base_init (gpointer base_class);
 
-struct GaleonEmbedPrivate
-{
-	gpointer dummy;
-};
-
 static guint galeon_embed_signals[LAST_SIGNAL] = { 0 };
 
 GType 
@@ -93,6 +78,7 @@ galeon_embed_get_type (void)
         return galeon_embed_type;
 }
 
+
 static void
 galeon_embed_base_init (gpointer g_class)
 {
@@ -102,18 +88,17 @@ galeon_embed_base_init (gpointer g_class)
     	{	
 	galeon_embed_signals[NEW_WINDOW] =
                 g_signal_new ("ge_new_window",
-                              GALEON_EMBED_TYPE,
-                              G_SIGNAL_RUN_FIRST,
+                              GALEON_TYPE_EMBED,
+                              G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, new_window),
-                              NULL, NULL,
-                              galeon_marshal_VOID__POINTER_INT,
-			      G_TYPE_NONE,
-                              2,
-                              G_TYPE_POINTER,
+                              _galeon_embed_new_window_signal_accumulator, NULL,
+                              galeon_marshal_OBJECT__INT,
+			      G_TYPE_OBJECT,
+                              1,
 			      G_TYPE_INT);
 	galeon_embed_signals[LINK_MESSAGE] =
                 g_signal_new ("ge_link_message",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, link_message),
                               NULL, NULL,
@@ -123,7 +108,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_STRING);
 	galeon_embed_signals[JS_STATUS] =
                 g_signal_new ("ge_js_status",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, js_status),
                               NULL, NULL,
@@ -133,7 +118,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_STRING);
 	galeon_embed_signals[LOCATION] =
                 g_signal_new ("ge_location",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, location),
                               NULL, NULL,
@@ -142,7 +127,7 @@ galeon_embed_base_init (gpointer g_class)
                               0);
 	galeon_embed_signals[TITLE] =
                 g_signal_new ("ge_title",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, title),
                               NULL, NULL,
@@ -151,7 +136,7 @@ galeon_embed_base_init (gpointer g_class)
                               0);
 	galeon_embed_signals[PROGRESS] =
                 g_signal_new ("ge_progress",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, progress),
                               NULL, NULL,
@@ -163,7 +148,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_INT);	
 	galeon_embed_signals[NET_STATE] =
                 g_signal_new ("ge_net_state",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, net_state),
                               NULL, NULL,
@@ -174,7 +159,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_INT);
 	galeon_embed_signals[VISIBILITY] =
                 g_signal_new ("ge_visibility",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, visibility),
                               NULL, NULL,
@@ -184,7 +169,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_BOOLEAN);
 	galeon_embed_signals[DESTROY_BRSR] =
                 g_signal_new ("ge_destroy_brsr",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, destroy_brsr),
                               NULL, NULL,
@@ -193,7 +178,7 @@ galeon_embed_base_init (gpointer g_class)
                               0);
 	galeon_embed_signals[OPEN_URI] =
                 g_signal_new ("ge_open_uri",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, open_uri),
                               NULL, NULL,
@@ -203,7 +188,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_STRING);
 	galeon_embed_signals[SIZE_TO] =
                 g_signal_new ("ge_size_to",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, size_to),
                               NULL, NULL,
@@ -214,27 +199,27 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_INT);
 	galeon_embed_signals[DOM_MOUSE_DOWN] =
                 g_signal_new ("ge_dom_mouse_down",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, dom_mouse_down),
-                              NULL, NULL,
-                              galeon_marshal_INT__OBJECT,
-                              G_TYPE_INT,
+			      g_signal_accumulator_true_handled, NULL,
+                              galeon_marshal_BOOLEAN__OBJECT,
+                              G_TYPE_BOOLEAN,
                               1,
-			      G_TYPE_POINTER);
+			      GALEON_TYPE_EMBED_EVENT);
 	galeon_embed_signals[DOM_MOUSE_CLICK] =
                 g_signal_new ("ge_dom_mouse_click",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, dom_mouse_click),
-                              NULL, NULL,
-                              galeon_marshal_INT__OBJECT,
-                              G_TYPE_INT,
+			      g_signal_accumulator_true_handled, NULL,
+                              galeon_marshal_BOOLEAN__OBJECT,
+                              G_TYPE_BOOLEAN,
                               1,
-			      G_TYPE_POINTER);
+			      GALEON_TYPE_EMBED_EVENT);
 	galeon_embed_signals[SECURITY_CHANGE] =
                 g_signal_new ("ge_security_change",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, security_change),
                               NULL, NULL,
@@ -244,7 +229,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_INT);
 	galeon_embed_signals[ZOOM_CHANGE] =
                 g_signal_new ("ge_zoom_change",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, zoom_change),
                               NULL, NULL,
@@ -254,7 +239,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_INT);
 	galeon_embed_signals[FAVICON] =
                 g_signal_new ("ge_favicon",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, favicon),
                               NULL, NULL,
@@ -264,7 +249,7 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_STRING);
 	galeon_embed_signals[POPUP_BLOCKED] =
                 g_signal_new ("ge_popupblocked",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, popupblocked),
                               NULL, NULL,
@@ -274,115 +259,171 @@ galeon_embed_base_init (gpointer g_class)
 			      G_TYPE_STRING);
 	galeon_embed_signals[CONTEXTMENU] =
                 g_signal_new ("ge_contextmenu",
-                              GALEON_EMBED_TYPE,
+                              GALEON_TYPE_EMBED,
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GaleonEmbedClass, contextmenu),
                               NULL, NULL,
                               g_cclosure_marshal_VOID__OBJECT,
                               G_TYPE_NONE,
                               1,
-			      GALEON_EMBED_EVENT_TYPE);
+			      GALEON_TYPE_EMBED_EVENT);
+	galeon_embed_signals[NET_STOP] =
+                g_signal_new ("ge_net_stop",
+                              GALEON_TYPE_EMBED,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, net_stop),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+	galeon_embed_signals[NET_START] =
+                g_signal_new ("ge_net_start",
+                              GALEON_TYPE_EMBED,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, net_start),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+/**
+ * GaleonEmbed::ge-content-change:
+ * @embed:
+ * @uri: URI of the new content
+ *
+ * The ::ge_content_change signal is emitted when a new page content
+ * is being loaded into the browser. It's a good place to do view
+ * related changes, for example to restore the zoom level of a page
+ * or to set an user style sheet.
+ **/
+		g_signal_new ("ge_content_change",
+			      GALEON_TYPE_EMBED,
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GaleonEmbedClass, content_change),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_STRING);
+
+/**
+ * GaleonEmbed::ge-modal-alert:
+ * @embed:
+ *
+ * The ::ge-modal-alert signal is emitted when a DOM event will open a
+ * modal alert.
+ *
+ * Return %TRUE to prevent the dialog from being opened.
+ **/
+		g_signal_new ("ge_modal_alert",
+			      GALEON_TYPE_EMBED,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GaleonEmbedClass, modal_alert),
+			      g_signal_accumulator_true_handled, NULL,
+			      galeon_marshal_BOOLEAN__VOID,
+			      G_TYPE_BOOLEAN,
+			      0);
+/**
+ * GaleonEmbed::ge-modal-alert-closed:
+ * @embed:
+ *
+ * The ::ge-modal-alert-closed signal is emitted when a modal alert put up by a
+ * DOM event was closed.
+ **/
+		g_signal_new ("ge_modal_alert_closed",
+			      GALEON_TYPE_EMBED,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GaleonEmbedClass, modal_alert_closed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
+/**
+ * GaleonEmbed::ge-show-js-console
+ * @embed:
+ *
+ * The ::ge-show-js-console signal is emitted when the rendering engine wants
+ * the javascript console to be shown.
+ *
+ */
+		g_signal_new ("ge_show_js_console",
+			      GALEON_TYPE_EMBED,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GaleonEmbedClass, show_js_console),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 	initialized = TRUE;
 	}
 }
 
-GaleonEmbed *
-galeon_embed_new (GObject *shell)
-{
-#ifdef ENABLE_MOZILLA_EMBED
-	if (IS_MOZILLA_EMBED_SHELL (shell))
-	{	
-		return GALEON_EMBED (g_object_new 
-			(MOZILLA_EMBED_TYPE, NULL));
-	}
-#endif
-	
-#ifdef ENABLE_GTKHTML_EMBED
-	if (IS_GTKHTML_EMBED_SHELL (shell))
-	{
-		return GALEON_EMBED (g_object_new 
-			(GTKHTML_EMBED_TYPE, NULL));
-	}
-#endif
-
-	g_assert_not_reached ();
-
-	return NULL;
-}
-
-void 
-galeon_embed_get_capabilities (GaleonEmbed *embed,
-                               EmbedCapabilities *caps)
-{
-	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        klass->get_capabilities (embed, caps);
-}
-
-gresult 
+void
 galeon_embed_load_url (GaleonEmbed *embed, 
                        const char *url)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->load_url (embed, url);
+        klass->load_url (embed, url);
 }
 
-gresult 
+void
 galeon_embed_stop_load (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->stop_load (embed);
+        klass->stop_load (embed);
 }
 
-gresult 
+gboolean
 galeon_embed_can_go_back (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
         return klass->can_go_back (embed);
 }
 
-gresult 
+gboolean 
 galeon_embed_can_go_forward (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
         return klass->can_go_forward (embed);
 }
 
-gresult 
+gboolean 
 galeon_embed_can_go_up (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
         return klass->can_go_up (embed);
 }
 
-gresult 
-galeon_embed_get_go_up_list (GaleonEmbed *embed, GSList **l)
+GSList *
+galeon_embed_get_go_up_list (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->get_go_up_list (embed, l);
+        return klass->get_go_up_list (embed);
 }
 
-gresult 
+void
 galeon_embed_go_back (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->go_back (embed);
+        klass->go_back (embed);
 }
 
-gresult  
+void
 galeon_embed_go_forward (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->go_forward (embed);
+        klass->go_forward (embed);
 }
 
-gresult
+void
 galeon_embed_go_up (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->go_up (embed);
+        klass->go_up (embed);
 }
 
-gresult
+gboolean
 galeon_embed_render_data (GaleonEmbed *embed, 
                           const char *data,
                           guint32 len,
@@ -393,7 +434,7 @@ galeon_embed_render_data (GaleonEmbed *embed,
         return klass->render_data (embed, data, len, base_uri, mime_type);
 }
 
-gresult
+gboolean
 galeon_embed_open_stream (GaleonEmbed *embed,
                           const char *base_uri,
                           const char *mime_type)
@@ -402,148 +443,135 @@ galeon_embed_open_stream (GaleonEmbed *embed,
         return klass->open_stream (embed, base_uri, mime_type);
 }
 
-gresult
+void
 galeon_embed_append_data (GaleonEmbed *embed,
                           const char *data, 
                           guint32 len)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->append_data (embed, data, len);
+        klass->append_data (embed, data, len);
 }
 
-gresult
+void
 galeon_embed_close_stream (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->close_stream (embed);
+        klass->close_stream (embed);
 }
 
-gresult
-galeon_embed_get_title (GaleonEmbed *embed,
-                        char **title)
+char *
+galeon_embed_get_title (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->get_title (embed, title);
+        return klass->get_title (embed);
 }
 
-gresult 
+char *
 galeon_embed_get_location (GaleonEmbed *embed, 
                            gboolean toplevel,
-                           gboolean requested,
-                           char **location)
+                           gboolean requested)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->get_location (embed, toplevel, requested, location);
+        return klass->get_location (embed, toplevel, requested);
 }
 
-gresult 
+void
 galeon_embed_reload (GaleonEmbed *embed, 
                      EmbedReloadFlags flags)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->reload (embed, flags);
+        klass->reload (embed, flags);
 }
 
-gresult
+void
 galeon_embed_copy_page (GaleonEmbed *dest,
 			GaleonEmbed *source,
 			EmbedDisplayType display_type)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (dest);
-	return klass->copy_page (dest, source, display_type);
+	klass->copy_page (dest, source, display_type);
 }	
 
-gresult 
-galeon_embed_grab_focus (GaleonEmbed *embed)
-{
-	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->grab_focus (embed);
-}
-
-gresult
+GList *
 galeon_embed_get_link_tags (GaleonEmbed *embed,
-                            const char *link_type,
-                            GList **tags)
+                            const char *link_type)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->get_link_tags (embed, link_type, tags);
+        return klass->get_link_tags (embed, link_type);
 }
 
-gresult
+void
 galeon_embed_zoom_set (GaleonEmbed *embed, 
-                       int zoom, 
-                       gboolean reflow)
+                       int zoom)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->zoom_set (embed, zoom, reflow);
+        klass->zoom_set (embed, zoom);
 }
 
-gresult
-galeon_embed_zoom_get (GaleonEmbed *embed,
-                       int *zoom)
+int
+galeon_embed_zoom_get (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->zoom_get (embed, zoom);
+        return klass->zoom_get (embed);
 }
 
-gresult 
+gboolean 
 galeon_embed_selection_can_cut (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
         return klass->selection_can_cut (embed);
 }
 
-gresult 
+gboolean 
 galeon_embed_selection_can_copy (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
         return klass->selection_can_copy (embed);
 }
 
-gresult 
+gboolean 
 galeon_embed_can_paste (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
         return klass->can_paste (embed);
 }
 
-gresult 
+void
 galeon_embed_select_all (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->select_all (embed);
+        klass->select_all (embed);
 }
 
-gresult
+void
 galeon_embed_selection_cut (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->selection_cut (embed);
+        klass->selection_cut (embed);
 }
 
-gresult
+void
 galeon_embed_selection_copy (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->selection_copy (embed);
+        klass->selection_copy (embed);
 }
 
-gresult
+void
 galeon_embed_paste (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->paste (embed);
+        klass->paste (embed);
 }
 
-gresult
-galeon_embed_shistory_count  (GaleonEmbed *embed,
-                              int *count)
+int
+galeon_embed_shistory_count  (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->shistory_count (embed, count);
+        return klass->shistory_count (embed);
 }
 
-gresult
+gboolean
 galeon_embed_shistory_get_nth (GaleonEmbed *embed, 
                                int nth,
                                gboolean is_relative,
@@ -554,15 +582,14 @@ galeon_embed_shistory_get_nth (GaleonEmbed *embed,
         return klass->shistory_get_nth (embed, nth, is_relative, url, title);
 }
 
-gresult
-galeon_embed_shistory_get_pos (GaleonEmbed *embed,
-                               int *pos)
+int
+galeon_embed_shistory_get_pos (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->shistory_get_pos (embed, pos);
+        return klass->shistory_get_pos (embed);
 }
 
-gresult
+gboolean
 galeon_embed_shistory_go_nth (GaleonEmbed *embed, 
                               int nth)
 {
@@ -570,7 +597,7 @@ galeon_embed_shistory_go_nth (GaleonEmbed *embed,
         return klass->shistory_go_nth (embed, nth);
 }
 
-gboolean
+void
 galeon_embed_shistory_copy (GaleonEmbed *source,
                             GaleonEmbed *dest,
 			    gboolean back_history,
@@ -578,26 +605,33 @@ galeon_embed_shistory_copy (GaleonEmbed *source,
 			    gboolean set_current)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (source);
-        return klass->shistory_copy (source, dest, back_history, forward_history, set_current);
+        klass->shistory_copy (source, dest, back_history, forward_history, set_current);
 }
 
-gresult
+void
+galeon_embed_shistory_clear (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	klass->shistory_clear (embed);
+}
+
+void
 galeon_embed_scroll (GaleonEmbed *embed, 
                      EmbedScrollDirection direction)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->scroll (embed, direction);
+        klass->scroll (embed, direction);
 }
 
-gresult
+void
 galeon_embed_fine_scroll (GaleonEmbed *embed, 
 			  int horiz, int vert)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->fine_scroll (embed, horiz, vert);
+        klass->fine_scroll (embed, horiz, vert);
 }
 
-gresult
+gboolean
 galeon_embed_get_security_level (GaleonEmbed *embed, 
                                  EmbedSecurityLevel *level,
                                  char **description)
@@ -606,23 +640,63 @@ galeon_embed_get_security_level (GaleonEmbed *embed,
         return klass->get_security_level (embed, level, description);
 }
 
-gresult
-galeon_embed_find (GaleonEmbed *embed,
-                   EmbedFindInfo *info)
+void
+galeon_embed_find_set_properties (GaleonEmbed *embed,
+				  char *search_string,
+				  gboolean case_sensitive,
+				  gboolean wrap_around)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->find (embed, info);
+        klass->find_set_properties (embed, search_string, 
+				    case_sensitive, wrap_around);
 }
 
-gresult
-galeon_embed_set_charset (GaleonEmbed *embed,
-                          const char *charset)
+gboolean
+galeon_embed_find_next (GaleonEmbed *embed,
+			gboolean backwards)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->set_charset (embed, charset);
+        return klass->find_next (embed, backwards);
 }
 
-gresult
+
+gboolean
+galeon_embed_set_encoding (GaleonEmbed *embed,
+			   const char *encoding)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+        return klass->set_encoding (embed, encoding);
+}
+
+GaleonEncodingPageInfo *
+galeon_embed_get_encoding_info (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+        return klass->get_encoding_info (embed);
+}
+
+gboolean
+galeon_embed_has_modified_forms (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	return klass->has_modified_forms (embed);
+}
+
+gboolean
+galeon_embed_can_view_source (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	return klass->can_view_source (embed);
+}
+
+void
+galeon_embed_evaluate_javascript (GaleonEmbed *embed, const char *script)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	klass->evaluate_javascript (embed, script);
+}
+
+gboolean
 galeon_embed_print (GaleonEmbed *embed, 
                     EmbedPrintInfo *info)
 {
@@ -630,21 +704,21 @@ galeon_embed_print (GaleonEmbed *embed,
         return klass->print (embed, info);
 }
 
-gresult
+void
 galeon_embed_print_preview_close (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->print_preview_close (embed);
+        klass->print_preview_close (embed);
 }
 
-gresult
-galeon_embed_print_preview_num_pages (GaleonEmbed *embed, gint *retNum)
+int
+galeon_embed_print_preview_num_pages (GaleonEmbed *embed)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-	return klass->print_preview_num_pages (embed, retNum);
+	return klass->print_preview_num_pages (embed);
 }	
 
-gresult
+gboolean
 galeon_embed_print_preview_navigate (GaleonEmbed *embed,
 				     EmbedPrintPreviewNavType navType,
 				     gint pageNum)
@@ -653,3 +727,101 @@ galeon_embed_print_preview_navigate (GaleonEmbed *embed,
 	return klass->print_preview_navigate (embed, navType, pageNum);
 }
 
+EmbedPageProperties *
+galeon_embed_get_page_properties(GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->get_page_properties (embed);
+}
+
+void
+galeon_embed_show_page_certificate (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	klass->show_page_certificate (embed);
+}
+
+GList *
+galeon_embed_get_stylesheets(GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->get_stylesheets (embed);
+}
+
+EmbedStyleSheet *
+galeon_embed_get_selected_stylesheet(GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->get_selected_stylesheet (embed);
+}
+
+void
+galeon_embed_set_stylesheet(GaleonEmbed *embed, EmbedStyleSheet *sheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	klass->set_stylesheet(embed, sheet);
+}
+
+gboolean
+galeon_embed_apply_user_stylesheet(GaleonEmbed *embed, const gchar *sheetfile,
+				   EmbedStyleSheet **retSheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->apply_user_stylesheet(embed, sheetfile, retSheet);
+}
+
+void
+galeon_embed_remove_user_stylesheet(GaleonEmbed *embed, EmbedStyleSheet *sheet)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	klass->remove_user_stylesheet(embed, sheet);
+}
+
+/* galeon_embed_stylesheet_new defined in mozilla-embed.cpp */
+/* galeon_embed_stylesheet_free defined in mozilla-embed.cpp */
+
+void
+galeon_embed_stylesheet_list_free (GList *l)
+{
+	GList *li;
+	for (li = l; li; li = li->next)
+	{
+		galeon_embed_stylesheet_free (li->data);
+	}
+	g_list_free (l);
+}
+
+gboolean
+galeon_embed_stylesheet_compare	(EmbedStyleSheet *ess1, EmbedStyleSheet *ess2)
+{
+	if (ess2 == ess1)
+	{
+		return TRUE;
+	}
+	else if (ess1 && ess2)
+	{
+		if (ess1->name && ess2->name && !strcmp (ess1->name, ess2->name))
+		{
+			return TRUE;
+		}
+		if (ess2->type != ess1->type
+		    || ess2->sheet != ess1->sheet)
+		{
+			return FALSE;
+		}
+		else if ((ess1->name == ess2->name) || !ess1->name || !ess2->name)
+			 
+		{
+			return TRUE;
+		}
+		else
+		{
+			return TRUE;
+		}
+	}
+	else
+	{
+		return FALSE;
+	}
+}
+
