@@ -18,7 +18,7 @@
 
 /*
  * The functioning of the download architecture, as described by Philip
- * on 28 May 2001 and updated on 28 June 2001:
+ * on 28 May 2001 and updated on 28 June 2001, and again on 05 Jan 2005:
  * 
  * When mozilla runs into a file it cannot render internally or that it
  * does not have a plugin for, it calls the
@@ -36,7 +36,10 @@
  * mozilla-the-browser. However, for dealing with helper apps, we do not
  * use the mozilla backend at all. This is because we want to use the
  * gnome-vfs database to retrieve helper-app info, rather than the
- * mozilla helper app database.
+ * mozilla helper app database. Note that mozilla now uses gnome-vfs for
+ * it's backend, so in theory, we don't need this step. In reality, I 
+ * remain suspicious of gnome-vfs's mime database and frankly don't trust
+ * any other code to work around its stupidities correctly :-)
  * 
  * How it works:
  * 
@@ -46,40 +49,30 @@
  * nsIHelperAppDialog.
  * 
  * b) In our implementation of Show() we first compare the passed mime
- * type to galeon's mime list. If the mime type is in the list, we then
- * lookup the Action associated with the mime type. Currently, the
- * possible mime-actions are:
+ * type to gnome-vfs's mime list. If any helpers are available, we build
+ * a list of them and offer the user the ability to open the file or save
+ * it to disk. If there are no helpers, we just offer the option of saving
+ * or cancelling the downlod.
  * 
- * Save to Disk
- * Run with Helper App
- * Ask User
- * 
- * The default action is Ask User, and if the mime-type is not in our
- * list, this is what will be assumed.
- * 
- * c) If Ask User is the chosen action, a dialog will be shown to the
- * user allowing the user to choose from the other two possible actions
- * as well as a checkbox to let the user set the default action to the
- * chosen action for the future.
- * 
- * d-1) The "Save to Disk" action. We first check galeon preferences to
- * see if the user wants to use the built-in mozilla downloader, gtm or
- * a command-line executed downloader.
+ * c-1) The "Save to Disk" action. We first check galeon preferences to
+ * see if the user wants to use the built-in mozilla downloader, or a
+ * command-line executed downloader. We still have our gtm code but until
+ * gtm is updated to use bonobo-activation, we can't use it effectively.
  *
- * d-2a) The built-in downloader.  This action is handled by the mozilla
+ * c-2a) The built-in downloader.  This action is handled by the mozilla
  * backend. Our nsIHelperAppDialog does the same thing that the
  * mozilla-the-browser one does, which is to call the SaveToDisk() method
  * of nsIExternalHelperAppService. This in turn calls the
  * PromptForSaveToFile() method of nsIHelperAppDialog putting the ball
  * back in our court.
  * 
- * d-2b) Now, if galeon is configured to always ask for a download
+ * c-2b) Now, if galeon is configured to always ask for a download
  * directory, it will pop up a file selector so that the user can select
  * the directory and filename to save the file to.  Otherwise, it will
  * use galeon's default download directory and proceed without
  * interaction.
  * 
- * d-2c) When PromptForSaveToFile() returns, nsIExternalHelperAppService
+ * c-2c) When PromptForSaveToFile() returns, nsIExternalHelperAppService
  * will then call the ShowProgressDialog() method of
  * nsIHelperAppDialog. This progress dialog, obviously, tracks the
  * progress of the download. It is worth noting that mozilla starts the
@@ -95,34 +88,26 @@
  * the download starts early means that the file is actually downloaded
  * to a temp file and only at the end is it moved to it's final location.
  * 
- * d-3a) The two external downloader options.  These options are
+ * c-3a) The two external downloader options.  These options are
  * handled completely by galeon. The first thing that we do is call the
  * Cancel() method of nsIExternalHelperAppService to cancel the mozilla
  * download. We then pass the url to our own LaunchExternalDownloader()
  * method. This method will ask for a download directory as appropriate
  * as with the "Save to disk" action.
  * 
- * d-3b) Finally, depending on whether GTM or a command line handler was
+ * c-3b) Finally, depending on whether GTM or a command line handler was
  * selected in prefs, the external handler will be called with the url
  * passed and the directory selected.
  * 
- * e-1) The "Run with Helper App" action.  This action is currently only
- * working with a minimal implementation.  First, we explicitly call
- * ShowProgressDialog() so the user knows that the file is being
- * downloaded. We also need this so that we only run the helper after the
- * file is completely downloaded. The file will download to temp location
- * that it would be moved from if the action was "Save to Disk".  We have
- * to call ShowProgressDialog() ourselves because we are not using
- * mozilla's helper mechanism which would usually make the call for us.
- * 
- * e-2) If there is a default helper app in our mime database and alwaysAsk
- * is false, galeon will run the default helper automatically. Otherwise it
- * will pop up a helper chooser dialog which lists the helpers that gnome-vfs
- * knows about as well as providing a GnomeFileEntry to allow the user to
- * select and arbitrary application. The default value of the GnomeFileEntry
- * is the helper stored in our database if one exits.
- * 
- * f) General notes.  We cannot use this infrastructure to override
+ * d-1) The "Run with Helper App" action. This implementation is slightly
+ * subversive with respect to the mozilla helper db but allows us to avoid
+ * inventing an information passing mechanism between ContentHandler and
+ * ProgressListener. Rather, we populate the mozilla MIME Info struct with
+ * the handler we retrieved from gnome-vfs but explicitly tell mozilla not
+ * to launch the helper. The ProgressListener retrieves this info when the
+ * download completes and then runs the helper.
+ *
+ * e) General notes.  We cannot use this infrastructure to override
  * native mozilla types. mozilla will attempt to render these types and
  * never call out to us. We are at the end of the chain as the handler of
  * last resort, so native and plugin types will never reach us. This also
@@ -132,26 +117,16 @@
  * 
  * Despite the apparent user-side similarity with explicit downloads by
  * a shift-click or context-menu item, there is actually none at all.
- * Explicit downloads are handled by the nsIStreamTransfer manager which
- * we use as is. Currently the progress dialog for the stream transfer
- * manager is un-overridable, so it appears in XUL. This will change in
- * due course.
+ * Explicit downloads are handled by nsIWebBrowserPersist. We reuse the
+ * progress dialog for these but nothing in ContentHandler is involved.
  * 
- * Matt would like the modifiy the progress dialog so each file currently
- * being downloaded becomes a clist entry in a master dialog rather than
- * causing a separate progress dialog. a lot of progress dialogs gets
- * really messy.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-extern "C" {
-#include "libgnomevfs/gnome-vfs-mime-handlers.h"
-}
 
-#include "galeon-embed-shell.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
 #include "gul-download.h"
@@ -160,48 +135,42 @@ extern "C" {
 #include "gul-general.h"
 #include "gul-gui.h"
 #include "galeon-embed-utils.h"
-#include "ContentHandler.h"
 #include "gul-file-chooser.h"
+#include "galeon-debug.h"
+#include "hig-alert.h"
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtkprogress.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkwindow.h>
-#include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkexpander.h>
-#include <libgnome/gnome-exec.h>
-#include <glib/gi18n.h>
-#include <libgnome/gnome-config.h>
-#include <libgnome/gnome-util.h>
+#include <gtk/gtkmain.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
-#include "MozillaPrivate.h"
-#include "ProgressListener.h"
+#include "ContentHandler.h"
+#include "GaleonUtils.h"   /* GaleonUtils::FindGtkParent */
+#include "ProgressListener.h" /* for BuildDownloadPath */
+#include "GulString.h"
 
-#include "nsCRT.h"
-#include "nsCOMPtr.h"
-#include "nsISupportsArray.h"
-#include "nsIServiceManager.h"
-#include "nsWeakReference.h"
-#include "nsIInterfaceRequestorUtils.h"
-
-#include "nsString.h"
-#include "nsIURI.h"
-#include "nsIURL.h"
-#include "nsIMIMEInfo.h"
-#include "nsIChannel.h"
-#include "nsIFTPChannel.h"
-#include "nsILocalFile.h"
-#include "nsIPrefService.h"
-#include "nsIDOMWindow.h"
-#include "nsIDOMWindowInternal.h"
+#include <nsIPromptService.h>
+#include <nsIWebProgressListener.h>
+#include <nsCOMPtr.h>
+#include <nsIServiceManager.h>
+#include <nsIInterfaceRequestorUtils.h>
+#include <nsIURI.h>
+#include <nsIURL.h>
+#include <nsIMIMEInfo.h>
+#include <nsILocalFile.h>
+#include <nsIDOMWindow.h>
+#include <nsIExternalHelperAppService.h>
+#include <nsCExternalHandlerService.h>
+#include <nsMemory.h>
 
 class GContentHandler;
 struct MimeAskActionDialog;
-struct HelperAppChooserDialog;
 
 extern "C"
 void mime_ask_dialog_save_clicked_cb (GtkButton *button,
@@ -224,14 +193,12 @@ struct MimeAskActionDialog
 			    const char *aMimeType);
 	~MimeAskActionDialog();
 
-	GContentHandler *mContentHandler;
-	GladeXML *mGXml;
+	GContentHandler* mContentHandler;
 	GtkWidget *mDialogWidget;
 	GtkWidget *mParent;
 	GtkWidget *mListView;
 
 	GList *mApps;
-	GtkListStore *mListStore;
 
 	enum
 	{
@@ -243,17 +210,43 @@ struct MimeAskActionDialog
 	static const int NUM_APPLICATIONS = 4; 
 };
 
+
+/* ------------------------------------------------------------
+ * Mini progress listener so that we can listen for errors
+ * while downloading */
+class MiniProgressListener : public nsIWebProgressListener
+{
+public:
+	MiniProgressListener(MimeAskActionDialog *dialog) : 
+		mDialog( dialog ) 
+	{ LOG ("MiniProgresListener ctor(%p)", this ); };
+
+	~MiniProgressListener() 
+	{ LOG ("MiniProgresListener dtor(%p)", this ); };
+
+ 	NS_DECL_ISUPPORTS
+	NS_DECL_NSIWEBPROGRESSLISTENER
+	
+	MimeAskActionDialog *mDialog;
+};
+
+NS_IMPL_ISUPPORTS1(MiniProgressListener, nsIWebProgressListener)
+
+// ------------------------------------------------------------
+
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
 GContentHandler::GContentHandler() : mUri(nsnull)
 {
 	/* member initializers and constructor code */
+	LOG ("GContentHandler ctor(%p)", this );
 }
 
 GContentHandler::~GContentHandler()
 {
 	/* destructor code */
+	LOG ("GContentHandler dtor(%p)", this );
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -262,12 +255,8 @@ GContentHandler::~GContentHandler()
 
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
-				    nsISupports *aContext
-#if MOZILLA_SNAPSHOT > 9
-, PRBool aForced)
-#else
-)
-#endif
+				    nsISupports *aContext,
+				    PRBool aForced)
 {
 	/* aForced reflects if the content being sent is normally viewable
 	 * in mozilla or not. That fact doesn't affect us, so ignore it
@@ -285,9 +274,7 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 
 /* nsILocalFile promptForSaveToFile (in nsISupports aWindowContext, in wstring aDefaultFile, in wstring aSuggestedFileExtension); */
 NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
-#if MOZILLA_SNAPSHOT > 10
 						   nsIHelperAppLauncher *aLauncher,
-#endif
 						   nsISupports *aWindowContext,
 						   const PRUnichar *aDefaultFile,
 						   const PRUnichar *aSuggestedFileExtension,
@@ -296,12 +283,12 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 	mContext = aWindowContext;
 
 	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (aWindowContext);
-	GtkWidget *parentWindow = GTK_WIDGET (MozillaFindGtkParent (parentDOMWindow));
+	GtkWidget *parentWindow = GaleonUtils::FindGtkParent (parentDOMWindow);
 
 	/* Get the directory to use for downloading into */
 	if (!eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
 	{
-		nsresult rv =  BuildDownloadPath (NS_ConvertUCS2toUTF8 (aDefaultFile).get(),
+		nsresult rv =  BuildDownloadPath (GulCString (aDefaultFile).get(),
 						  parentWindow, _retval);
 		if (NS_SUCCEEDED (rv)) return rv;
 	}
@@ -311,7 +298,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 					GTK_FILE_CHOOSER_ACTION_SAVE,
 					CONF_STATE_LAST_DOWNLOAD_DIR);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
-					   NS_ConvertUCS2toUTF8 (aDefaultFile).get());
+					   GulCString (aDefaultFile).get());
 
 	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
 	{
@@ -330,7 +317,7 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 
 		nsCOMPtr <nsILocalFile> destFile;
 
-		NS_NewNativeLocalFile (nsDependentCString (filename),
+		NS_NewNativeLocalFile (GulDependentCString (filename),
 				       PR_TRUE,
 				       getter_AddRefs(destFile));
 		g_free (filename);
@@ -346,16 +333,6 @@ NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
 	return NS_ERROR_FAILURE;
 }
 
-#if MOZILLA_SNAPSHOT <= 9
-/* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
-NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
-					      nsISupports *aContext)
-{
-	g_print ("GContentHandler::ShowProgressDialog is depreciated!\n");
-	return NS_ERROR_NOT_IMPLEMENTED;
-}
-#endif
-
 ////////////////////////////////////////////////////////////////////////////////
 // begin local public methods impl
 ////////////////////////////////////////////////////////////////////////////////
@@ -401,20 +378,16 @@ NS_METHOD GContentHandler::LaunchHelperApp (void)
 			appLauncher->DeleteTemporaryFileOnExit(mTempFile);
 		}
 
-		nsString uFileName;
+		GulString uFileName;
 		mTempFile->GetPath(uFileName);
-		const nsCString &aFileName = NS_ConvertUCS2toUTF8(uFileName);
 
-		const nsCString &document = (mUrlHelper) ? mUrl : aFileName;
-
-		char *param = g_strdup (document.get());
-		gul_general_launch_application (mHelperApp->command,
-						param,
-						mHelperApp->requires_terminal);
+		/* FIXME: check return value */
+		gul_general_launch_application (mHelperApp,
+						mUrlHelper ? mUrl.get()
+						: GulCString (uFileName).get(),
+						gtk_get_current_event_time());
 
 		if(mUrlHelper) mLauncher->Cancel();
-
-		g_free (param);
 	}
 	else
 	{
@@ -436,32 +409,45 @@ NS_METHOD GContentHandler::GetContext (nsISupports * *_retval)
 	return NS_OK;
 }
 
-static gboolean 
-application_support_scheme (GnomeVFSMimeApplication *app, const nsCString &aScheme)
-{
-	GList *l;
 
-	g_return_val_if_fail (app != NULL, FALSE);
-	g_return_val_if_fail (!aScheme.IsEmpty(), FALSE);
+NS_METHOD GContentHandler::CheckAppSupportScheme (void)
+{
+	g_return_val_if_fail (mHelperApp, NS_ERROR_FAILURE);
+
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+
+	/* There is no way to get the supported schemes in the new API */
+	mUrlHelper = gnome_vfs_mime_application_supports_uris (mHelperApp);
+#else
+	mUrlHelper = FALSE;
+
+	if (mHelperApp->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
+	{
+		return NS_OK;
+	}
 	
-	if (app->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
-		return FALSE;
-	
-	for (l = app->supported_uri_schemes; l != NULL; l = l->next)
+	GList *l;
+	for (l = mHelperApp->supported_uri_schemes; l != NULL; l = l->next)
 	{
 		char *uri_scheme = (char *)l->data;
 		g_return_val_if_fail (uri_scheme != NULL, FALSE);
-		if (aScheme.Equals(uri_scheme)) return TRUE;
+		if (mScheme.Equals (uri_scheme))
+		{
+			mUrlHelper = TRUE;
+			return NS_OK;
+		}
 	}
+#endif
 
-	return FALSE;
+	return NS_OK;
 }
 
 NS_METHOD GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 					PRBool alwaysUse)
 {
 	mHelperApp = aHelperApp;
-	mUrlHelper = application_support_scheme (aHelperApp, mScheme);
+
+	CheckAppSupportScheme();
 
 	return NS_OK;
 }
@@ -473,34 +459,32 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	rv = mLauncher->GetMIMEInfo(getter_AddRefs(mimeInfo));
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	gchar *command = g_find_program_in_path (mHelperApp->command);
-	if (command == NULL) return NS_ERROR_FAILURE;
+	// XXX information passing kludge (to ProgressListener)
+	// TODO: should probably use some prefix, like 'gnomevfs:'
+	
+	const char * id;
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+	id = gnome_vfs_mime_application_get_desktop_id (mHelperApp);
+#else
+	id = mHelperApp->id;
+#endif
+
+	/* The current time is fine here as the user has just clicked
+	 * a button (it is used as the time for the application opening) */
+	char * info = g_strdup_printf ("%d:%s", gtk_get_current_event_time(),
+				       id);
+
+	rv = mimeInfo->SetApplicationDescription
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+		(GulString(info));
+#else
+		(GulString(info).get());
+#endif
+	g_free (info);
 
-	nsCOMPtr<nsILocalFile> helperFile;
-	rv = NS_NewNativeLocalFile(nsDependentCString(command),
-				   PR_TRUE,
-				   getter_AddRefs(helperFile));
-	g_free (command);
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	rv = mimeInfo->SetPreferredApplicationHandler(helperFile);
-	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;	
-
-	nsMIMEInfoHandleAction mimeInfoAction;
-	mimeInfoAction = nsIMIMEInfo::useHelperApp;
-
-	if(mHelperApp->requires_terminal) //Information passing kludge!
-	{
-		rv = mimeInfo->SetApplicationDescription
-#if MOZILLA_SNAPSHOT > 16
-				(NS_LITERAL_STRING("runInTerminal"));
-#else
-				(NS_LITERAL_STRING("runInTerminal").get());
-#endif
-		if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
-	}
-
-	rv = mimeInfo->SetPreferredAction(mimeInfoAction);
+	rv = mimeInfo->SetPreferredAction(nsIMIMEInfo::useHelperApp);
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 	return NS_OK;
@@ -515,22 +499,17 @@ NS_METHOD GContentHandler::Init (void)
 
 	nsCOMPtr<nsIMIMEInfo> MIMEInfo;
 	rv = mLauncher->GetMIMEInfo (getter_AddRefs(MIMEInfo));
-#if MOZILLA_SNAPSHOT > 16
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
 	rv = MIMEInfo->GetMIMEType (mMimeType);
 #else
 	char *type = nsnull;
 	rv = MIMEInfo->GetMIMEType (&type);
-	mMimeType.Adopt (type);
+	mMimeType = type;
+	nsMemory::Free (type);
 #endif
 	
-#if MOZILLA_SNAPSHOT > 11
 	rv = mLauncher->GetSource(getter_AddRefs(mUri));
 	rv = mLauncher->GetTargetFile(getter_AddRefs(mTempFile));
-#else
-	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
-					&mTimeDownloadStarted,
-					getter_AddRefs(mTempFile));
-#endif
 
 	rv = mUri->GetSpec (mUrl);
 	rv = mUri->GetScheme (mScheme);
@@ -545,17 +524,22 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 	if (mMimeType.IsEmpty() ||
 	    mMimeType.Equals("application/octet-stream"))
 	{
-		nsresult rv;
-		nsCOMPtr<nsIURL> url = do_QueryInterface(mUri, &rv);
-		if (NS_SUCCEEDED(rv) && url)
-		{
-			nsCAutoString uriFileName;
-			url->GetFileName(uriFileName);
-			mMimeType.Assign(gnome_vfs_mime_type_from_name
-						(uriFileName.get()));
-		}
-		else
-			mMimeType.Assign("application/octet-stream");
+		// suggested filename
+		GulCString uriFileName;
+
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+		GulString filename;
+		mLauncher->GetSuggestedFileName (filename);
+		uriFileName = filename;
+#else
+		PRUnichar *filename = nsnull;
+		mLauncher->GetSuggestedFileName(&filename);
+		uriFileName = filename;
+		nsMemory::Free(filename);
+#endif
+
+		mMimeType.Assign(gnome_vfs_mime_type_from_name
+				 (uriFileName.get()));
 	}
 
 	return NS_OK;
@@ -564,9 +548,18 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 NS_METHOD GContentHandler::MIMEAskAction (void)
 {
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface(mContext);
-	GtkWidget *parentWidget = MozillaFindGtkParent(parent);
+	GtkWidget *parentWidget = GaleonUtils::FindGtkParent(parent);
 
-	new MimeAskActionDialog(this, parentWidget, mMimeType.get());
+	MimeAskActionDialog *dialog =
+		new MimeAskActionDialog(this, parentWidget, mMimeType.get());
+
+	/* Create a mini progress listener while the mime ask action
+	 * dialog is being displayed. Using this listener, we can catch
+	 * errors while the download is happening to a temporary file
+	 * - bug 148739
+	 */
+	nsCOMPtr<nsIWebProgressListener> listener = new MiniProgressListener( dialog );
+	mLauncher->SetWebProgressListener( listener );
 
 	return NS_OK;
 }
@@ -582,16 +575,20 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 					 mParent(aParentWidget)
 {
 	GtkWidget *label;
+	GladeXML *GXml;
 	char *markup;
+	LOG ("MimeAskActionDialog ctor(%p)", this );
 
 	mApps = gnome_vfs_mime_get_all_applications(aMimeType);
 	if (mApps)
 	{
-		mGXml = gul_glade_widget_new("galeon.glade",
+		GtkListStore * liststore;
+
+		GXml = gul_glade_widget_new("galeon.glade",
 					     "mime_ask_open_dialog", 
 					     &mDialogWidget, this);
 
-		mListView = glade_xml_get_widget(mGXml, "mime_ask_dialog_helper_list_view");
+		mListView = glade_xml_get_widget(GXml, "mime_ask_dialog_helper_list_view");
 
 		GnomeVFSMimeApplication *defaultApp =
 			gnome_vfs_mime_get_default_application(aMimeType);
@@ -609,15 +606,15 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 		markup = g_strdup_printf ("<span size=\"larger\" weight=\"bold\">%s</span>", title);
 		g_free (title);
 
-		label = glade_xml_get_widget(mGXml, "mime_ask_action_title");
+		label = glade_xml_get_widget(GXml, "mime_ask_action_title");
 		gtk_label_set_markup (GTK_LABEL(label), markup);
 		g_free (markup);
 
 
-		mListStore = gtk_list_store_new(N_COLUMNS,
+		liststore = gtk_list_store_new(N_COLUMNS,
 						G_TYPE_STRING);
 		gtk_tree_view_set_model(GTK_TREE_VIEW(mListView),
-					GTK_TREE_MODEL(mListStore));
+					GTK_TREE_MODEL(liststore));
 
 		GtkCellRenderer *renderer;
 		renderer = gtk_cell_renderer_text_new();
@@ -640,8 +637,8 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 				static_cast<GnomeVFSMimeApplication *>(i->data);
 
 			GtkTreeIter iter;
-			gtk_list_store_append (mListStore, &iter);
-			gtk_list_store_set (mListStore, &iter,
+			gtk_list_store_append (liststore, &iter);
+			gtk_list_store_set (liststore, &iter,
 					    APP_COLUMN, app->name,
 					    -1);
 
@@ -652,6 +649,8 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 			}
 		}
 
+		g_object_unref (liststore);
+
 		// Calculate the correct width for the list view for showing
 		// NUM_APPLICATION rows.
 		gint width, height, ypad;
@@ -663,15 +662,15 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 	}
 	else
 	{
-		mGXml = gul_glade_widget_new("galeon.glade",
+		GXml = gul_glade_widget_new("galeon.glade",
 					     "mime_ask_save_dialog", 
 					     &mDialogWidget, this);
 		GtkWidget *w;
-		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_save");
+		w = glade_xml_get_widget(GXml, "mime_ask_dialog_save");
 		gtk_widget_grab_default (w);
 	}
 
-	GtkWidget *mimeIcon = glade_xml_get_widget(mGXml,
+	GtkWidget *mimeIcon = glade_xml_get_widget(GXml,
 						   "mime_ask_action_icon");
 	gul_gui_image_set_from_mime_type (mimeIcon, 
 			                  aMimeType, GTK_ICON_SIZE_DIALOG);
@@ -681,18 +680,21 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 	(void)mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
 	// suggested filename
-#if MOZILLA_SNAPSHOT <= 16
-	PRUnichar *filename = nsnull;
-	launcher->GetSuggestedFileName(&filename);
-	markup = g_strdup_printf ("<b>%s</b>", NS_ConvertUCS2toUTF8(filename).get());
-	nsMemory::Free(filename);
+	GulCString filename;
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+	GulString uFilename;
+	launcher->GetSuggestedFileName (uFilename);
+	filename = uFilename;
 #else
-        nsAutoString filename;
-        launcher->GetSuggestedFileName (filename);
-	markup = g_strdup_printf ("<b>%s</b>", NS_ConvertUCS2toUTF8(filename).get());
+	PRUnichar *uFilename = nsnull;
+	launcher->GetSuggestedFileName(&uFilename);
+	filename = uFilename;
+	nsMemory::Free(uFilename);
 #endif
 
-	label = glade_xml_get_widget(mGXml, "mime_ask_action_name");
+	markup = g_markup_printf_escaped("<b>%s</b>", filename.get());
+
+	label = glade_xml_get_widget(GXml, "mime_ask_action_name");
 	gtk_label_set_markup(GTK_LABEL(label), markup);
 	g_free (markup);
 
@@ -700,18 +702,18 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 	const char *description = gnome_vfs_mime_get_description(aMimeType);
 	if (!description) description = aMimeType;
 	
-	label = glade_xml_get_widget(mGXml, "mime_ask_action_description");
+	label = glade_xml_get_widget(GXml, "mime_ask_action_description");
 	gtk_label_set_text(GTK_LABEL(label), description);
 
 	// source
 	nsCOMPtr<nsIURI> source;
 	launcher->GetSource(getter_AddRefs(source));
 
-	nsCAutoString spec;
+	GulCString spec;
 	source->Resolve(NS_LITERAL_CSTRING("."), spec);
 
-	markup = g_strdup_printf ("<i>%s</i>", PromiseFlatCString(spec).get());
-	label = glade_xml_get_widget(mGXml, "mime_ask_action_source");
+	markup = g_strdup_printf ("<i>%s</i>", spec.get());
+	label = glade_xml_get_widget(GXml, "mime_ask_action_source");
 	gtk_label_set_markup(GTK_LABEL(label), markup);
 	g_free (markup);
 
@@ -719,15 +721,18 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 				     GTK_WINDOW(aParentWidget));
 
 	gtk_widget_show(mDialogWidget);
+
+	g_object_unref(G_OBJECT(GXml));
 }
 
 MimeAskActionDialog::~MimeAskActionDialog()
 {
+	LOG ("MimeAskActionDialog dtor(%p)", this );
+
 	if(mApps)
 		gnome_vfs_mime_application_list_free(mApps);
 
 	gtk_widget_destroy(mDialogWidget);
-	g_object_unref(G_OBJECT(mGXml));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -739,9 +744,10 @@ mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 {
 	gtk_widget_hide(dialog->mDialogWidget);
 
-	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
-	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+	dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+
+	launcher->SetWebProgressListener( nsnull );
 	
  	if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER)
  	    == DOWNLOADER_BUILTIN)
@@ -760,10 +766,10 @@ mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 static void
 mime_ask_dialog_download_cancel (MimeAskActionDialog *dialog)
 {
-	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
-	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+	dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
+	launcher->SetWebProgressListener( nsnull );
 	launcher->Cancel();
 
 	delete dialog;
@@ -772,9 +778,8 @@ mime_ask_dialog_download_cancel (MimeAskActionDialog *dialog)
 extern "C" void
 mime_ask_dialog_open_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 {
-	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
-	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(launcher));
+	dialog->mContentHandler->GetLauncher (getter_AddRefs(launcher));
 
 	GnomeVFSMimeApplication *app(NULL);
 	if (dialog->mApps)
@@ -797,6 +802,7 @@ mime_ask_dialog_open_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 
 	if (app)
 	{
+		launcher->SetWebProgressListener( nsnull );
 		dialog->mContentHandler->SetHelperApp(app, FALSE);
 		dialog->mContentHandler->FindHelperApp();
 		delete dialog;
@@ -832,6 +838,67 @@ mime_ask_dialog_close_cb(GtkWidget *dialogWidget, MimeAskActionDialog *dialog)
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
 	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
+	launcher->SetWebProgressListener( nsnull );
 	launcher->Cancel();
 }
 
+// ------------------------------------------------------------
+
+NS_IMETHODIMP MiniProgressListener::OnStatusChange(nsIWebProgress *aWebProgress, 
+					       nsIRequest *aRequest, nsresult aStatus, 
+					       const PRUnichar *aMessage)
+{
+	if (NS_SUCCEEDED (aStatus)) return NS_OK;
+
+	/* When the save fails, the mozilla code will call this, and
+	 * after this point, the launcher isn't valid for some reason
+	 * so we need to display an error, and get out of this code
+	 * quickly */
+	GtkWidget *dialog = 
+		hig_alert_new (GTK_WINDOW (mDialog->mParent), 
+			       (GtkDialogFlags)0,
+			       HIG_ALERT_ERROR,
+			       ("Failed to Save File."),
+			       GulCString (aMessage).get(),
+			       GTK_STOCK_OK, GTK_RESPONSE_OK,
+			       NULL);
+
+	delete mDialog; /* Close, and delete the MIME dialog */
+	mDialog = NULL;
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	return NS_OK;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnStateChange(nsIWebProgress *aWebProgress, 
+					      nsIRequest *aRequest, PRUint32 aStateFlags, 
+					      nsresult aStatus)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnProgressChange(nsIWebProgress *aWebProgress, 
+						 nsIRequest *aRequest, PRInt32 aCurSelfProgress,
+						 PRInt32 aMaxSelfProgress, PRInt32 aCurTotalProgress,
+						 PRInt32 aMaxTotalProgress)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnLocationChange(nsIWebProgress *aWebProgress, 
+						 nsIRequest *aRequest, nsIURI *location)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnSecurityChange(nsIWebProgress *aWebProgress, 
+						 nsIRequest *aRequest, PRUint32 state)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+
+
+
