@@ -22,25 +22,15 @@
 #include "mozilla-embed-shell.h"
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include "nsIHelperAppLauncherDialog.h"
-#include "nsIExternalHelperAppService.h"
-#include "nsCExternalHandlerService.h"
-#include "nsIWebProgressListener.h"
 
-#include "nsString.h"
-#include "nsIURI.h"
-#include "nsILocalFile.h"
+#include <nsIHelperAppLauncherDialog.h>
+#include <nsCOMPtr.h>
 
-#include "nsCOMPtr.h"
-#include "nsISupports.h"
-#include "nsError.h"
+#include "GulString.h"
 
-typedef enum
-{
-        ACTION_NONE,
-        ACTION_SAVEFORHELPER,
-        ACTION_OBJECT_NOTIFY
-} DownloadAction;
+/* forward declarations */
+class nsIURI;
+class nsIFile;
 
 #define G_CONTENTHANDLER_CID			     \
 { /* 16072c4a-23a6-4996-9beb-9335c06bbeae */         \
@@ -64,7 +54,6 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	NS_METHOD LaunchExternalDownloader (void);
 	NS_METHOD FindHelperApp (void);
 	NS_METHOD LaunchHelperApp (void);
-	NS_METHOD ShowHelperProgressDialog (void);
 
 	NS_METHOD GetLauncher (nsIHelperAppLauncher * *_retval);
 	NS_METHOD GetContext (nsISupports * *_retval);
@@ -77,7 +66,8 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	NS_METHOD Init (void);
 	NS_METHOD ProcessMimeInfo (void);
 	NS_METHOD MIMEAskAction (void);
-	
+	NS_METHOD CheckAppSupportScheme (void);
+
 	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
 	nsCOMPtr<nsISupports> mContext;
 
@@ -85,17 +75,12 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	PRInt64 mTimeDownloadStarted;
 	nsCOMPtr<nsIFile> mTempFile;
 
-	char *mMimeType;	
+	GulCString mMimeType;
 	PRBool mUrlHelper;
 	GnomeVFSMimeApplication *mHelperApp;
 	
-	nsCString mUrl;
-	nsCString mScheme;
-
-	PRBool mDownloadCanceled;
-	PRBool mHelperProgress;
-	
-	nsCOMPtr<nsIWebProgressListener> mListener;
+	GulCString mUrl;
+	GulCString mScheme;
 };
 
 #endif
