@@ -30,21 +30,27 @@
 
 #ifdef HAVE_MOZILLA_PSM
 
-#include "MozillaPrivate.h"
+#include "GaleonUtils.h"
 
-#include "nsIX509Cert.h"
-#include "nsCOMPtr.h"
-#include "nsIServiceManager.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIInterfaceRequestorUtils.h"
-#include "nsIX509CertValidity.h"
-#include "nsIX509CertDB.h"
-#include "nsIASN1Object.h"
-#include "nsIASN1Sequence.h"
-#include "nsICRLInfo.h"
-#include "nsISimpleEnumerator.h"
-#include "nsIArray.h"
-#include "nsString.h"
+
+#include <nsCOMPtr.h>
+#include <nsMemory.h>
+#include <nsIServiceManager.h>
+#include <nsIInterfaceRequestor.h>
+#include <nsIInterfaceRequestorUtils.h>
+#include <nsIX509Cert.h>
+#include <nsIX509CertValidity.h>
+#include <nsIX509CertDB.h>
+#include <nsIASN1Object.h>
+#include <nsIASN1Sequence.h>
+#include <nsICRLInfo.h>
+#include <nsISimpleEnumerator.h>
+#include <nsIArray.h>
+#define MOZILLA_STRICT_API 1
+#include <nsEmbedString.h>
+#undef MOZILLA_STRICT_API
+#include <nsIDOMWindow.h>
+#include <nsIServiceManager.h>
 
 #include <gconf/gconf-client.h>
 #include <gtk/gtkdialog.h>
@@ -204,14 +210,13 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 	int res;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
-        g_return_val_if_fail (GTK_IS_WINDOW (gparent), GTK_RESPONSE_CANCEL);
         g_return_val_if_fail (markup_text, GTK_RESPONSE_CANCEL);
         g_return_val_if_fail (!checkbox_text || checkbox_value, GTK_RESPONSE_CANCEL);
 	
 	dialog = gtk_dialog_new_with_buttons ("",
-					      GTK_WINDOW (gparent),
+					      gparent ? GTK_WINDOW (gparent) : NULL,
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      NULL);
 
@@ -279,17 +284,20 @@ GtkNSSDialogs::ConfirmMismatchDomain (nsIInterfaceRequestor *ctx,
                                       const nsACString &targetURL,
                                       nsIX509Cert *cert, PRBool *_retval)
 {
-	nsAutoString commonName;
 	char *ttTargetUrl, *ttCommonName, *first, *second, *msg;
 	int res;
 
+	nsEmbedString commonName;
 	cert->GetCommonName (commonName);
 
- 	ttTargetUrl = g_strdup_printf ("\"<tt>%s</tt>\"", 
-                                        PromiseFlatCString(targetURL).get());
+	nsEmbedCString cCommonName;
+	NS_UTF16ToCString (commonName,
+			   NS_CSTRING_ENCODING_UTF8, cCommonName);
 
-	ttCommonName = g_strdup_printf ("\"<tt>%s</tt>\"", 
-                                         NS_ConvertUCS2toUTF8(commonName).get());
+ 	ttTargetUrl = g_markup_printf_escaped ("\"<tt>%s</tt>\"", 
+                                               nsEmbedCString(targetURL).get());
+
+	ttCommonName = g_markup_printf_escaped ("\"<tt>%s</tt>\"", cCommonName.get());
 
         first = g_strdup_printf (_("The site %s returned security information for "
 				   "%s. It is possible that someone is intercepting "
@@ -324,17 +332,20 @@ GtkNSSDialogs::ConfirmUnknownIssuer (nsIInterfaceRequestor *ctx,
                                      PRBool *_retval)
 {
 	gboolean accept_perm = FALSE;
-	nsAutoString commonName;
 	char *ttCommonName, *secondary, *tertiary, *msg;
 	int res;
 
+	nsEmbedString commonName;
 	cert->GetCommonName (commonName);
 
-	ttCommonName = g_strdup_printf ("\"<tt>%s</tt>\"", 
-					NS_ConvertUCS2toUTF8(commonName).get());
+	nsEmbedCString cCommonName;
+	NS_UTF16ToCString (commonName,
+			   NS_CSTRING_ENCODING_UTF8, cCommonName);
+
+	ttCommonName = g_markup_printf_escaped ("\"<tt>%s</tt>\"", cCommonName.get());
 
 	secondary = g_strdup_printf
-		           (_("Your browser was unable to trust %s. "
+		           (_("It was not possible to automatically trust %s. "
 			      "It is possible that someone is intercepting your "
 			      "communication to obtain your confidential information."),
 			      ttCommonName);
@@ -389,7 +400,6 @@ GtkNSSDialogs::ConfirmCertExpired (nsIInterfaceRequestor *ctx,
 	PRTime now = PR_Now();
 	PRTime notAfter, notBefore, timeToUse;
 	PRInt64 normalizedTime;
-	nsAutoString commonName;
 	time_t t;
 	struct tm tm;
 	char formattedDate[128];
@@ -423,8 +433,13 @@ GtkNSSDialogs::ConfirmCertExpired (nsIInterfaceRequestor *ctx,
 		timeToUse = notBefore;
 	}
 	
+	nsEmbedString commonName;
 	cert->GetCommonName (commonName);
 
+	nsEmbedCString cCommonName;
+	NS_UTF16ToCString (commonName,
+			   NS_CSTRING_ENCODING_UTF8, cCommonName);
+
 	LL_DIV (normalizedTime, timeToUse, PR_USEC_PER_SEC);
 	LL_L2UI (t, normalizedTime);
 	/* To translators: this a time format that is used while displaying the
@@ -434,8 +449,7 @@ GtkNSSDialogs::ConfirmCertExpired (nsIInterfaceRequestor *ctx,
 		  localtime_r (&t, &tm));
 	fdate = g_locale_to_utf8 (formattedDate, -1, NULL, NULL, NULL);
 
-	ttCommonName = g_strdup_printf ("\"<tt>%s</tt>\"", 
-                                        NS_ConvertUCS2toUTF8(commonName).get());
+	ttCommonName = g_markup_printf_escaped ("\"<tt>%s</tt>\"", cCommonName.get());
 
 	secondary = g_strdup_printf (text, ttCommonName, fdate);
 
@@ -463,10 +477,9 @@ GtkNSSDialogs::NotifyCrlNextupdate (nsIInterfaceRequestor *ctx,
 {
 	GtkWidget *dialog, *label;
 	char *ttCommonName, *ttTargetUrl, *msg, *primary, *secondary;
-	nsAutoString commonName;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 	dialog = gtk_dialog_new_with_buttons ("",
 					      GTK_WINDOW (gparent),
@@ -478,13 +491,17 @@ GtkNSSDialogs::NotifyCrlNextupdate (nsIInterfaceRequestor *ctx,
 	higgy_setup_dialog (GTK_DIALOG (dialog), GTK_STOCK_DIALOG_ERROR,
 			    &label, NULL);
 
+	nsEmbedString commonName;
 	cert->GetCommonName (commonName);
 
-	ttCommonName = g_strdup_printf ("\"<tt>%s</tt>\"", 
-					NS_ConvertUCS2toUTF8(commonName).get());
+	nsEmbedCString cCommonName;
+	NS_UTF16ToCString (commonName,
+			   NS_CSTRING_ENCODING_UTF8, cCommonName);
 
-	ttTargetUrl = g_strdup_printf ("\"<tt>%s</tt>\"", 
-				       PromiseFlatCString(targetURL).get());
+	ttCommonName = g_markup_printf_escaped ("\"<tt>%s</tt>\"", cCommonName.get());
+
+	ttTargetUrl = g_markup_printf_escaped ("\"<tt>%s</tt>\"",
+				               nsEmbedCString(targetURL).get());
 
 	primary = g_strdup_printf (_("Cannot establish connection to %s."),
 				   ttTargetUrl);
@@ -525,7 +542,7 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	char *ttCommonName, *msg, *tertiary;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 	dialog = gtk_dialog_new_with_buttons ("",
 					      GTK_WINDOW (gparent),
@@ -540,12 +557,16 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 			    &label, &content_vbox);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
-	nsAutoString commonName;
+	nsEmbedString commonName;
 	cert->GetCommonName (commonName);
-	ttCommonName = g_strdup_printf ("\"<tt>%s</tt>\"", 
-					 NS_ConvertUCS2toUTF8(commonName).get());
 
-	tertiary = g_strdup_printf (_("Trust %s to identify:"), ttCommonName );
+	nsEmbedCString cCommonName;
+	NS_UTF16ToCString (commonName,
+			   NS_CSTRING_ENCODING_UTF8, cCommonName);
+
+	ttCommonName = g_markup_printf_escaped ("\"<tt>%s</tt>\"",  cCommonName.get());
+
+	tertiary = g_strdup_printf (_("Trust %s to identify:"), ttCommonName);
 	g_free (ttCommonName);
 
 	msg = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
@@ -618,7 +639,7 @@ GtkNSSDialogs::NotifyCACertExists (nsIInterfaceRequestor *ctx)
 	char * msg;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 	dialog = gtk_dialog_new_with_buttons ("",
 					      GTK_WINDOW (gparent),
@@ -738,7 +759,7 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	char *msg;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 
 	dialog = gtk_dialog_new_with_buttons ("",
@@ -830,8 +851,9 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	}
 	else
 	{
-		gchar * text = gtk_editable_get_chars (GTK_EDITABLE (entry1), 0, -1);
-		_password = NS_ConvertUTF8toUCS2 (text);
+		gchar *text = gtk_editable_get_chars (GTK_EDITABLE (entry1), 0, -1);
+		NS_CStringToUTF16 (nsEmbedCString (text),
+			           NS_CSTRING_ENCODING_UTF8, _password);
 		g_free (text);
 		*_retval = PR_TRUE;
 	}
@@ -848,7 +870,7 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	char *msg;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 	dialog = gtk_dialog_new_with_buttons ("",
 					      GTK_WINDOW (gparent),
@@ -891,7 +913,8 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	else
 	{
 		gchar * text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-		_password = NS_ConvertUTF8toUCS2 (text);
+		NS_CStringToUTF16 (nsEmbedCString (text),
+			           NS_CSTRING_ENCODING_UTF8, _password);
 		g_free (text);
 		*_retval = PR_TRUE;
 	}
@@ -929,7 +952,7 @@ GtkNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl
 	char *msg;
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 
 	dialog = gtk_dialog_new_with_buttons ("",
 					      GTK_WINDOW (gparent),
@@ -950,7 +973,7 @@ GtkNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
-	nsAutoString org, orgUnit, nextUpdate;
+	nsEmbedString org, orgUnit, nextUpdate;
 	rv = crl->GetOrganization (org);
 	if (NS_FAILED(rv)) return rv;
 
@@ -960,13 +983,19 @@ GtkNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl
 	rv = crl->GetNextUpdateLocale (nextUpdate);
 	if (NS_FAILED(rv)) return rv;
 
-	label = gtk_label_new (NS_ConvertUCS2toUTF8(org).get());
+	nsEmbedCString cOrg;
+	NS_UTF16ToCString (org, NS_CSTRING_ENCODING_UTF8, cOrg);
+	label = gtk_label_new (cOrg.get());
 	set_table_row (table, 0, _("Organization:"), label);
 
-	label = gtk_label_new (NS_ConvertUCS2toUTF8(orgUnit).get());
+	nsEmbedCString cOrgUnit;
+	NS_UTF16ToCString (orgUnit, NS_CSTRING_ENCODING_UTF8, cOrgUnit);
+	label = gtk_label_new (cOrgUnit.get());
 	set_table_row (table, 1, _("Unit:"), label);
 
-	label = gtk_label_new (NS_ConvertUCS2toUTF8(nextUpdate).get());
+	nsEmbedCString cNextUpdate;
+	NS_UTF16ToCString (nextUpdate, NS_CSTRING_ENCODING_UTF8, cNextUpdate);
+	label = gtk_label_new (cNextUpdate.get());
 	set_table_row (table, 2, _("Next Update:"), label);
 
 	gtk_box_pack_start (GTK_BOX (vbox), higgy_indent_widget (table), FALSE, FALSE, 0);
@@ -984,14 +1013,14 @@ GtkNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl
  *  Help function to fill in the labels on the General tab
  */
 static void
-set_label_cert_attribute (GladeXML* gxml, const char* label_id, nsAutoString &value)
+set_label_cert_attribute (GladeXML* gxml, const char* label_id, nsAString &value)
 {
 	GtkWidget *label;
 	label = glade_xml_get_widget (gxml, label_id);
 
 	g_return_if_fail (GTK_IS_LABEL (label));
 
-	if (value.IsEmpty()) {
+	if (!value.Length()) {
 		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 		char *msg = g_strdup_printf ("<i>&lt;%s&gt;</i>",
 					     _("Not part of certificate"));
@@ -1000,9 +1029,10 @@ set_label_cert_attribute (GladeXML* gxml, const char* label_id, nsAutoString &va
 	}
 	else
 	{
+		nsEmbedCString cValue;
+		NS_UTF16ToCString (value, NS_CSTRING_ENCODING_UTF8, cValue);
 		gtk_label_set_use_markup (GTK_LABEL (label), FALSE);
-		gtk_label_set_text (GTK_LABEL (label),
-				    NS_ConvertUCS2toUTF8(value).get());
+		gtk_label_set_text (GTK_LABEL (label), cValue.get());
 	}
 }
 
@@ -1032,15 +1062,18 @@ fill_cert_chain_tree (GtkTreeView *treeview, nsIArray *certChain)
 		gtk_tree_store_append (GTK_TREE_STORE (model), &iter, 
 				       (i == (int)numCerts-1) ? NULL : &parent);
 
-		nsAutoString value;
+		nsEmbedString value;
 		rv = nsCert->GetCommonName (value);
 		if (NS_FAILED(rv)) return FALSE;
 
+		nsEmbedCString cValue;
+		NS_UTF16ToCString (value, NS_CSTRING_ENCODING_UTF8, cValue);
+
 		nsIX509Cert *nsCertP = nsCert;
-		if (!value.IsEmpty())
+		if (value.Length())
 		{
 			gtk_tree_store_set (GTK_TREE_STORE(model), &iter,
-					    0, NS_ConvertUCS2toUTF8(value).get(),
+					    0, cValue.get(),
 					    1, nsCertP,
 					    -1);
 		}
@@ -1074,14 +1107,17 @@ fill_cert_chain_tree (GtkTreeView *treeview, nsIArray *certChain)
 static void
 add_asn1_object_to_tree(GtkTreeModel *model, nsIASN1Object *object, GtkTreeIter *parent)
 {
-	nsAutoString dispNameU;
+	nsEmbedString dispNameU;
 	object->GetDisplayName(dispNameU);
 
+	nsEmbedCString cDispNameU;
+	NS_UTF16ToCString (dispNameU, NS_CSTRING_ENCODING_UTF8, cDispNameU);
+
 	GtkTreeIter iter;
 	gtk_tree_store_append (GTK_TREE_STORE (model), &iter, parent);
 
 	gtk_tree_store_set (GTK_TREE_STORE(model), &iter,
-			    0, NS_ConvertUCS2toUTF8(dispNameU).get(),
+			    0, cDispNameU.get(),
 			    1, object,
 			    -1);
 
@@ -1150,14 +1186,16 @@ field_tree_view_selection_changed_cb (GtkTreeSelection *selection,
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 	{
 		nsIASN1Object *object;
-		nsAutoString dispValU;
 
 		gtk_tree_model_get (model, &iter, 1, &object, -1);
 
+		nsEmbedString dispValU;
 		object->GetDisplayValue(dispValU);
-		gtk_text_buffer_set_text (text_buffer, 
-					  NS_ConvertUCS2toUTF8(dispValU).get(),
-					  -1);
+
+		nsEmbedCString cDispValU;
+		NS_UTF16ToCString (dispValU, NS_CSTRING_ENCODING_UTF8, cDispValU);
+
+		gtk_text_buffer_set_text (text_buffer, cDispValU.get(), -1);
 	}
 	else
 	{
@@ -1186,6 +1224,7 @@ setup_view_cert_tree (GtkWidget *dialog, GladeXML*gxml, nsIArray *certChain)
 						  G_TYPE_STRING,
 						  G_TYPE_POINTER);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (chain_tree_view), GTK_TREE_MODEL (store));
+	g_object_unref (store);
 
 
 	renderer = gtk_cell_renderer_text_new ();
@@ -1207,6 +1246,7 @@ setup_view_cert_tree (GtkWidget *dialog, GladeXML*gxml, nsIArray *certChain)
 				    G_TYPE_STRING,
 				    G_TYPE_POINTER);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (field_tree_view), GTK_TREE_MODEL (store));
+	g_object_unref (store);
 
 
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (field_tree_view),
@@ -1247,7 +1287,7 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 {
 	GtkWidget *dialog, *widget;
 	GladeXML *gxml;
-	nsAutoString value;
+	nsEmbedString value;
 	nsresult rv;
 	PRUint32 verifystate, count;
 	PRUnichar ** usage;
@@ -1258,7 +1298,7 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 				     &dialog, NULL);
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
-	GtkWidget *gparent = MozillaFindGtkParent (parent);
+	GtkWidget *gparent = GaleonUtils::FindGtkParent (parent);
 	gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(gparent));
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 
@@ -1310,11 +1350,11 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 		text = _("Could not verify this certificate for unknown reasons.");
 	}
 	
-	char *msg = g_strdup_printf ("<b>%s</b>", text);
+	char *vmsg = g_strdup_printf ("<b>%s</b>", text);
 	widget = glade_xml_get_widget (gxml, "label_verify_text");
 	g_return_val_if_fail (GTK_IS_LABEL (widget), NS_ERROR_FAILURE);
-	gtk_label_set_markup (GTK_LABEL (widget), msg);
-	g_free (msg);
+	gtk_label_set_markup (GTK_LABEL (widget), vmsg);
+	g_free (vmsg);
 
 	if (count > 0)
 	{
@@ -1322,8 +1362,11 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 		GtkWidget *indent;
 		for (PRUint32 i = 0 ; i < count ; i++)
 		{
-			const nsACString &msg = NS_ConvertUCS2toUTF8 (usage[i]);
-			GtkWidget *label = gtk_label_new(PromiseFlatCString(msg).get());
+			nsEmbedCString msg;
+			NS_UTF16ToCString (nsEmbedString(usage[i]),
+					   NS_CSTRING_ENCODING_UTF8, msg);
+
+			GtkWidget *label = gtk_label_new(msg.get());
 			gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 			gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 			nsMemory::Free (usage[i]);
