@@ -24,6 +24,7 @@
 
 #include "gul-general.h"
 #include "eel-gconf-extensions.h"
+#include "egg-recent-model.h"
 #include "prefs-strings.h"
 #include "gul-gui.h"
 #include "galeon-debug.h"
@@ -35,36 +36,25 @@
 
 #include <nsIWebBrowserPersist.h>
 #include <nsCWebBrowserPersist.h>
-
 #include <nsIFileURL.h>
-#include "nsXPIDLString.h"
-#include "nsIFTPChannel.h"
-#include "nsIHttpChannel.h"
-#include "nsIMIMEInfo.h"
-#include "nsNetError.h"                 // for NS_BINDING_ABORTED, see FIXME
-#include "nsString.h"
+#include <nsIFTPChannel.h>
+#include <nsIHttpChannel.h>
+#include <nsIMIMEInfo.h>
+#include <nsNetError.h>                 // for NS_BINDING_ABORTED, see FIXME
+#include <nsILocalFile.h>
+#include <nsIObserver.h>
+#include <nsCExternalHandlerService.h>
+#include <nsMemory.h>
 
 static const PRInt32 kInterval            = 500000;     /* in microsecs == 500ms == 0.5s */
 static const PRInt32 kUnknownSizeInterval = 150000;     /* in microsecs == 150ms == 0.15s */
 
-NS_IMPL_ADDREF(GProgressListener);
-NS_IMPL_RELEASE(GProgressListener);
-
-/*
- * If nsISupports is mapped through nsIDownload, then aborting right-click
- * downloads will crash out. I assume it's because nsIDownload and nsIProgressDialog
- * both inherit from nsITransfer.
- */
-NS_INTERFACE_MAP_BEGIN(GProgressListener)
-	NS_INTERFACE_MAP_ENTRY(nsIDownload)
-	NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
-	NS_INTERFACE_MAP_ENTRY(nsIProgressDialog)
-	NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
-	NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIWebProgressListener)
-#if MOZILLA_SNAPSHOT > 15
-	NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsITransfer, nsIDownload)
+#ifdef HAVE_NSITRANSFER_H
+NS_IMPL_ISUPPORTS3(GProgressListener, nsIWebProgressListener, nsITransfer, nsISupportsWeakReference)
+#else
+NS_IMPL_ISUPPORTS4(GProgressListener, nsIWebProgressListener, 
+		   nsIDownload, nsITransfer, nsISupportsWeakReference)
 #endif
-NS_INTERFACE_MAP_END
 
 //---------------------------------------------------------------------------
 
@@ -95,10 +85,12 @@ GProgressListener::GProgressListener () : mPersist(nsnull),
 					  mObserver(nsnull),
 					  mMIMEInfo(nsnull)
 {
+	LOG ("GProgressListener ctor(%p)", this);
 }
 
 GProgressListener::~GProgressListener ()
 {
+	LOG ("GProgressListener dtor(%p)", this);
 }
 
 NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
@@ -124,6 +116,14 @@ NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
 	return rv;
 }
 
+static void
+download_weak_ref_cb (gpointer data, GObject *object)
+{
+	GProgressListener *progress = (GProgressListener*)data;
+	NS_RELEASE (progress);
+}
+
+
 NS_METHOD GProgressListener::PrivateInit (void)
 {
 	nsresult rv;
@@ -133,16 +133,18 @@ NS_METHOD GProgressListener::PrivateInit (void)
 	mIsPaused            = PR_FALSE;
 	mLastUpdate          = PR_Now ();
 
-	EmbedPersistFlags flags = EMBED_PERSIST_SHOW_PROGRESS;
+	guint flags = EMBED_PERSIST_ADD_TO_RECENT | EMBED_PERSIST_SHOW_PROGRESS;
         if (mGaleonPersist)
         {
                 g_object_get (mGaleonPersist, "flags", &flags, NULL);
         }
+
+	mAddToRecent = flags & EMBED_PERSIST_ADD_TO_RECENT;
  
         if (flags & EMBED_PERSIST_SHOW_PROGRESS)
 	{
-		nsAutoString dest;
-		nsCAutoString source;
+		GulString dest;
+		GulCString source;
 
 		rv = mFile->GetPath (dest);
 
@@ -151,7 +153,7 @@ NS_METHOD GProgressListener::PrivateInit (void)
 		DownloaderView *dv;
 		dv = galeon_embed_shell_get_downloader_view (embed_shell);
 		mDownload = downloader_view_add_download (dv, source.get(),
-							  NS_ConvertUCS2toUTF8(dest).get());
+							  GulCString (dest).get());
 		g_signal_connect (G_OBJECT (mDownload), "cancel",
 				  G_CALLBACK (download_cancel_cb), this);
 		g_signal_connect (G_OBJECT (mDownload), "pause",
@@ -159,6 +161,12 @@ NS_METHOD GProgressListener::PrivateInit (void)
 		g_signal_connect (G_OBJECT (mDownload), "resume",
 				  G_CALLBACK (download_resume_cb), this);
 		g_object_add_weak_pointer (G_OBJECT (mDownload), (void **) &mDownload);
+
+		/* Ensure that we stay around while the download is alive,
+		 * bug 144067 */
+		NS_ADDREF (this);
+		g_object_weak_ref (G_OBJECT (mDownload), download_weak_ref_cb,
+				   this);
 	}
 
 	/* done */
@@ -166,11 +174,7 @@ NS_METHOD GProgressListener::PrivateInit (void)
 }
 
 NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
-#if MOZILLA_SNAPSHOT > 15
 				      nsIURI *aTarget,
-#else
-                                      nsILocalFile *aTarget,
-#endif
                                       const PRUnichar *aDisplayName,
 				      nsIMIMEInfo *aMIMEInfo,
                                       PRInt64 aStartTime,
@@ -180,7 +184,6 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
         mStartTime = aStartTime;
         mPersist = aPersist;
 
-#if MOZILLA_SNAPSHOT > 15
 	mTarget = aTarget;
 	nsCOMPtr<nsIFileURL> tFileUrl = do_QueryInterface(mTarget);
 	if (tFileUrl)
@@ -189,9 +192,6 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
         	tFileUrl->GetFile(getter_AddRefs(tFileRef));
 		tFileRef->Clone(getter_AddRefs(mFile));
 	}
-#else
-        mFile = aTarget;
-#endif
 
 	if (aMIMEInfo)
 	{
@@ -212,11 +212,7 @@ NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
         return PrivateInit();
 }
 
-NS_IMETHODIMP GProgressListener::Open(nsIDOMWindow *aParent)
-{
-        return NS_OK;
-}
-
+#ifndef HAVE_NSITRANSFER_H
 /* attribute long long startTime; */
 NS_IMETHODIMP GProgressListener::GetStartTime(PRInt64 *aStartTime)
 {
@@ -224,6 +220,7 @@ NS_IMETHODIMP GProgressListener::GetStartTime(PRInt64 *aStartTime)
         return NS_OK;
 }
 
+
 /* attribute nsIURI source; */
 NS_IMETHODIMP GProgressListener::GetSource(nsIURI * *aSource)
 {
@@ -231,10 +228,6 @@ NS_IMETHODIMP GProgressListener::GetSource(nsIURI * *aSource)
         return NS_OK;
 }
 
-#if MOZILLA_SNAPSHOT <= 15
-/* attribute nsILocalFile target; */
-NS_IMETHODIMP GProgressListener::GetTarget(nsILocalFile * *aTarget)
-#else
 /* attribute nsIURI target; */
 NS_IMETHODIMP GProgressListener::GetTarget(nsIURI * *aTarget)
 {
@@ -244,7 +237,6 @@ NS_IMETHODIMP GProgressListener::GetTarget(nsIURI * *aTarget)
 
 /* attribute nsILocalFile targetFile; */
 NS_IMETHODIMP GProgressListener::GetTargetFile(nsILocalFile * *aTarget)
-#endif
 {
         nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(mFile);
         NS_IF_ADDREF(*aTarget = localFile);
@@ -257,6 +249,49 @@ NS_IMETHODIMP GProgressListener::GetMIMEInfo(nsIMIMEInfo * *aMIMEInfo)
 	return NS_OK;
 }
 
+/* readonly attribute PRInt32 percentComplete; */
+NS_IMETHODIMP GProgressListener::GetPercentComplete(PRInt32 *aPercentComplete)
+{
+	*aPercentComplete = 0;
+        return NS_OK;
+}
+
+#ifdef HAVE_NSIDOWNLOAD_GETSIZE
+/* readonly attribute PRUint64 amountTransferred; */
+NS_IMETHODIMP GProgressListener::GetAmountTransferred(PRUint64 *aAmountTransferred)
+{
+   *aAmountTransferred = 0;
+   return NS_OK;
+}
+
+
+/* readonly attribute PRUint64 size; */
+NS_IMETHODIMP GProgressListener::GetSize(PRUint64 *aSize)
+{
+   *aSize = 0;
+   return NS_OK;
+}
+#endif    
+
+/* attribute wstring displayName; */
+NS_IMETHODIMP GProgressListener::GetDisplayName(PRUnichar * *aDisplayName)
+{
+        *aDisplayName = nsnull;
+        return NS_OK;
+}
+NS_IMETHODIMP GProgressListener::SetDisplayName(const PRUnichar * aDisplayName)
+{
+        return NS_OK;
+}
+
+NS_IMETHODIMP GProgressListener::GetPersist(nsIWebBrowserPersist * *aPersist)
+{
+        NS_IF_ADDREF(*aPersist = mPersist);
+        return NS_OK;
+}
+
+#endif
+
 /* attribute nsIObserver observer; */
 NS_IMETHODIMP GProgressListener::GetObserver(nsIObserver * *aObserver)
 {
@@ -280,52 +315,8 @@ NS_IMETHODIMP GProgressListener::SetListener(nsIWebProgressListener * aListener)
         return NS_OK;
 }
 
-/* readonly attribute PRInt32 percentComplete; */
-NS_IMETHODIMP GProgressListener::GetPercentComplete(PRInt32 *aPercentComplete)
-{
-	*aPercentComplete = 0;
-        return NS_OK;
-}
-
-/* attribute wstring displayName; */
-NS_IMETHODIMP GProgressListener::GetDisplayName(PRUnichar * *aDisplayName)
-{
-        *aDisplayName = nsnull;
-        return NS_OK;
-}
-NS_IMETHODIMP GProgressListener::SetDisplayName(const PRUnichar * aDisplayName)
-{
-        return NS_OK;
-}
-
-NS_IMETHODIMP GProgressListener::GetPersist(nsIWebBrowserPersist * *aPersist)
-{
-        NS_IF_ADDREF(*aPersist = mPersist);
-        return NS_OK;
-}
-
-NS_IMETHODIMP GProgressListener::SetDialog(nsIDOMWindow *aDialog)
-{
-        return NS_OK;
-}
-
-NS_IMETHODIMP GProgressListener::GetDialog(nsIDOMWindow * *aDialog)
-{
-        *aDialog = nsnull;
-        return NS_OK;
-}
-
-/* attribute PRBool cancelDownloadOnClose; */
-NS_IMETHODIMP  GProgressListener::GetCancelDownloadOnClose(PRBool *aCancelDownloadOnClose)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-NS_IMETHODIMP  GProgressListener::SetCancelDownloadOnClose(PRBool aCancelDownloadOnClose)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-static void LaunchHandler (PersistHandlerInfo *aHandler, nsIFile *aFile)
+static void LaunchHandler (GnomeVFSMimeApplication *aHandler, nsIFile *aFile,
+			   guint32 user_time)
 {
 	nsCOMPtr<nsIExternalHelperAppService> helperService =
 		do_GetService (NS_EXTERNALHELPERAPPSERVICE_CONTRACTID);
@@ -337,12 +328,11 @@ static void LaunchHandler (PersistHandlerInfo *aHandler, nsIFile *aFile)
 		appLauncher->DeleteTemporaryFileOnExit(aFile);
 	}
 
-	nsAutoString filename;
+	GulString filename;
 	aFile->GetPath(filename);
 	
-	gul_general_launch_application (aHandler->command,
-				        NS_ConvertUCS2toUTF8(filename).get(),
-					aHandler->need_terminal);
+	/* FIXME: check return value */
+	gul_general_launch_application (aHandler, GulCString(filename).get(), user_time);
 }
 
 /*
@@ -389,7 +379,7 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
                 // keep us around forever.  Detaching would kill us instantly
                 // so kungfu is needed to survive just long enough to do the
                 // final clean up.
-                nsCOMPtr<nsIDownload> kungFuDeathGrip(this);
+                nsCOMPtr<nsITransfer> kungFuDeathGrip(this);
                 if (mPersist)
                 {
                         mPersist->SetProgressListener(nsnull);
@@ -398,6 +388,39 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
 		
 		if (NS_SUCCEEDED(aStatus))
 		{
+			if (mAddToRecent)
+			{
+				EggRecentModel *model;
+
+				model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_NONE);
+
+				GulString dest;
+				mFile->GetPath (dest);
+
+				EggRecentItem *item;
+			       
+				if (mContentType.IsEmpty() || 
+				    mContentType.EqualsAsciiIgnoreCase("application/octet-stream"))
+				{
+					item = egg_recent_item_new_from_uri (GulCString(dest).get());
+				}
+				else
+				{
+					item = egg_recent_item_new ();
+					egg_recent_item_set_uri (item, GulCString(dest).get());
+					egg_recent_item_set_mime_type (item, mContentType.get());
+				}
+
+				// keep in sync with PrintProgressListener.cpp
+				egg_recent_item_add_group (item, "Galeon");
+				egg_recent_item_add_group (item, "Web Browser");
+
+				egg_recent_model_add_full (model, item);
+				egg_recent_item_unref (item);
+
+				g_object_unref (model);
+			}
+
 			if (mDownload)
 			{
 				g_object_remove_weak_pointer (G_OBJECT(mDownload), (void **) &mDownload);
@@ -406,13 +429,15 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
 
 			if (G_IS_OBJECT(mGaleonPersist))
 			{
-				PersistHandlerInfo *handler(nsnull);
+				GnomeVFSMimeApplication *handler = NULL;
+				guint32 user_time;
 				g_object_get (mGaleonPersist,
-			      		      "handler", &handler,        
+			      		      "handler", &handler,
+					      "user_time", &user_time,
 			      		      NULL);
 				if (handler)
 				{
-					LaunchHandler (handler, mFile);
+					LaunchHandler (handler, mFile, user_time);
 				}
 
 				g_signal_emit_by_name (mGaleonPersist, "completed");
@@ -596,6 +621,15 @@ NS_METHOD GProgressListener::SetRequest (nsIRequest *aRequest)
 				download_set_size_total (mDownload, mContentLength);
 			}
 		}
+
+		if (mContentType.IsEmpty())
+		{
+			GulCString type;
+			if (NS_SUCCEEDED(channel->GetContentType(type)))
+			{
+				mContentType = type;
+			}
+		}
 	}
 
 	nsCOMPtr<nsIFTPChannel> ftp = do_QueryInterface(aRequest);
@@ -615,17 +649,39 @@ NS_METHOD GProgressListener::LaunchHelperApp (void)
 
 	nsresult rv;
 
-	nsCOMPtr<nsIFile> helperFile;
-	rv = mMIMEInfo->GetPreferredApplicationHandler(getter_AddRefs(helperFile));
+	GulCString cFileName;
+	rv = mFile->GetNativePath(cFileName);
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	nsCAutoString helperFileName;
-	rv = helperFile->GetNativePath(helperFileName);
+	GulString helperDesc;
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+	rv = mMIMEInfo->GetApplicationDescription(helperDesc);
+#else
+	PRUnichar *tmp;
+	rv = mMIMEInfo->GetApplicationDescription (&tmp);
+	helperDesc = tmp;
+	nsMemory::Free (tmp);
+#endif
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	nsMIMEInfoHandleAction mimeAction;
-	rv = mMIMEInfo->GetPreferredAction(&mimeAction);
-	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
+	// XXX information passing kludge (from ContentHandler)
+	// TODO: should probably use some prefix, like 'gnomevfs:'
+	GulCString helperId(helperDesc);
+
+	// Format <usertime>:<helperapp id>
+	char *colon = strchr (helperId.get(), ':');
+	g_return_val_if_fail (colon, NS_ERROR_FAILURE);
+
+	const char *helperappid = colon+1;
+
+	guint32 usertime = strtoul (helperId.get(), &colon, 0);
+	
+	GnomeVFSMimeApplication *helper = 
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+		gnome_vfs_mime_application_new_from_desktop_id (helperappid);
+#else
+		gnome_vfs_mime_application_new_from_id (helperappid);
+#endif
 
 	nsCOMPtr<nsIExternalHelperAppService> helperService =
 		do_GetService (NS_EXTERNALHELPERAPPSERVICE_CONTRACTID, &rv);
@@ -639,26 +695,9 @@ NS_METHOD GProgressListener::LaunchHelperApp (void)
 		}
 	}
 
-	nsCAutoString cFileName;
-	mFile->GetNativePath(cFileName);
-	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-#if MOZILLA_SNAPSHOT > 16
-	nsAutoString helperDesc;
-	rv = mMIMEInfo->GetApplicationDescription(helperDesc);
-#else
-	nsXPIDLString helperDesc;
-	rv = mMIMEInfo->GetApplicationDescription(getter_Copies(helperDesc));
-#endif
-	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-	gboolean terminalHelper = 
-		helperDesc.Equals(NS_LITERAL_STRING("runInTerminal")) ?
-		TRUE : FALSE;
-
-	gul_general_launch_application (helperFileName.get(),
-					cFileName.get(),
-					terminalHelper);
+	/* FIXME: check return value */
+	gul_general_launch_application (helper, cFileName.get(), usertime);
+	gnome_vfs_mime_application_free (helper);
 
 	return NS_OK;
 }
@@ -695,7 +734,7 @@ nsresult GProgressListener::Resume (void)
 nsresult GProgressListener::Abort (void)
 {
 	// Keep ourselves alive
-	nsCOMPtr<nsIDownload> kungFuDeathGrip(this);
+	nsCOMPtr<nsITransfer> kungFuDeathGrip(this);
 
 	// Suspended request doesn't seem to reach cancelled state until it's
 	// resumed.  Better resume it first so that we get the right away.
@@ -706,7 +745,7 @@ nsresult GProgressListener::Abort (void)
 
         if (mObserver)
         {
-                mObserver->Observe(NS_ISUPPORTS_CAST(nsIProgressDialog*, this),
+                mObserver->Observe(NS_ISUPPORTS_CAST(nsITransfer*, this),
 				   "oncancel", nsnull);
         }
 
@@ -760,7 +799,7 @@ InitiateMozillaDownload (nsIURI *sourceURI, nsILocalFile* inDestFile,
 	nsresult rv = NS_OK;
 
 	EmbedPersistFlags flags;
-	galeon_embed_persist_get_flags (GALEON_EMBED_PERSIST (embedPersist), &flags);
+	flags = galeon_embed_persist_get_flags (GALEON_EMBED_PERSIST (embedPersist));
 
 	nsCOMPtr<nsIWebBrowserPersist> webPersist = 
 		do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID, &rv);
@@ -800,11 +839,12 @@ InitiateMozillaDownload (nsIURI *sourceURI, nsILocalFile* inDestFile,
 		{
 			/* Create a directory to hold the content, mozilla
 			 * will create the directory as needed. */
-			nsCAutoString nativePath;
+			GulCString nativePath;
 			inDestFile->GetNativePath(nativePath);
 			
-			NS_NewNativeLocalFile(nativePath + NS_LITERAL_CSTRING("_content"),
-					      PR_TRUE, getter_AddRefs(filesFolder));
+			nativePath.Append ("_content");
+
+			NS_NewNativeLocalFile(nativePath, PR_TRUE, getter_AddRefs(filesFolder));
 		}
 
 		rv = webPersist->SaveDocument (domDocument, inDestFile, filesFolder,
@@ -852,7 +892,7 @@ BuildDownloadPath (const char *defaultFileName, GtkWidget *parent,
 	if (gul_gui_confirm_overwrite_file (parent, filename))
 	{
 		nsCOMPtr <nsILocalFile> saveFile;
-		NS_NewNativeLocalFile (nsDependentCString (filename),
+		NS_NewNativeLocalFile (GulDependentCString (filename),
 				       PR_TRUE,
 				       getter_AddRefs(saveFile));
 			
