@@ -23,12 +23,13 @@
 #include "galeon-action-navigation.h"
 #include "galeon-shell.h"
 #include "galeon-favicon-cache.h"
-#include "gul-gobject-misc.h"
 #include "gul-toolbutton.h"
 #include "gul-string.h"
+#include "galeon-debug.h"
 
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
 
 #include <glib/gi18n.h>
 #include <string.h>
@@ -41,8 +42,6 @@ enum {
 
 #define MENU_ITEM_MAX_LENGTH 40
 
-static void		galeon_action_navigation_init			(GaleonActionNavigation *action);
-static void		galeon_action_navigation_class_init		(GaleonActionNavigationClass *class);
 static void		galeon_action_navigation_set_property		(GObject *object,
 									 guint prop_id,
 									 const GValue *value,
@@ -56,8 +55,9 @@ static void		galeon_action_navigation_update			(GaleonAction *action);
 static void		galeon_action_navigation_current_embed_changed	(GaleonAction *action, 
 									 GaleonEmbed *old_embed, GaleonEmbed *new_embed);
 static void             galeon_action_navigation_activate               (GtkAction *action);
+static void             galeon_action_navigation_finalize               (GObject *object);
 
-static GtkActionClass *parent_class = NULL;
+G_DEFINE_TYPE (GaleonActionNavigation, galeon_action_navigation, GALEON_TYPE_ACTION);
 
 GType
 galeon_action_navigation_direcion_get_type (void)
@@ -76,11 +76,6 @@ galeon_action_navigation_direcion_get_type (void)
 	return etype;
 }
 
-MAKE_GET_TYPE (galeon_action_navigation, "GaleonActionNavigation", GaleonActionNavigation,
-	       galeon_action_navigation_class_init, 
-	       galeon_action_navigation_init, GALEON_TYPE_ACTION);
-
-
 static void
 galeon_action_navigation_class_init (GaleonActionNavigationClass *class)
 {
@@ -90,6 +85,7 @@ galeon_action_navigation_class_init (GaleonActionNavigationClass *class)
 
 	object_class->set_property = galeon_action_navigation_set_property;
 	object_class->get_property = galeon_action_navigation_get_property;
+	object_class->finalize = galeon_action_navigation_finalize;
 
 	action_class->menu_item_type    = GTK_TYPE_IMAGE_MENU_ITEM;
 	action_class->toolbar_item_type = GUL_TYPE_TOOLBUTTON;
@@ -116,13 +112,20 @@ galeon_action_navigation_class_init (GaleonActionNavigationClass *class)
 							    GALEON_NAVIGATION_DIRECTION_BACK,
 							    G_PARAM_READWRITE));
 	
-	parent_class = g_type_class_peek_parent (class);
 }
 
 static void
 galeon_action_navigation_init (GaleonActionNavigation *action)
 {
+	LOG ("GaleonActionNavigation ctor (%p)", action);
 }
+static void
+galeon_action_navigation_finalize (GObject *object)
+{
+	LOG ("GaleonActionNavigation dtor (%p)", object);
+	G_OBJECT_CLASS (galeon_action_navigation_parent_class)->finalize (object);
+}
+
 
 static void
 galeon_action_navigation_set_property (GObject *object,
@@ -185,7 +188,7 @@ galeon_action_navigation_update (GaleonAction *action)
 		switch (GALEON_ACTION_NAVIGATION (action)->direction)
 		{
 		case GALEON_NAVIGATION_DIRECTION_BACK:
-			sensitive = (galeon_embed_can_go_back (embed) == G_OK);
+			sensitive = galeon_embed_can_go_back (embed);
 			if (sensitive)
 			{
 				galeon_embed_shistory_get_nth (embed, -1, TRUE, &url, &title);
@@ -193,7 +196,7 @@ galeon_action_navigation_update (GaleonAction *action)
 			}
 			break;
 		case GALEON_NAVIGATION_DIRECTION_FORWARD:
-			sensitive = (galeon_embed_can_go_forward (embed) == G_OK);
+			sensitive = galeon_embed_can_go_forward (embed);
 			if (sensitive)
 			{
 				galeon_embed_shistory_get_nth (embed, +1, TRUE, &url, &title);
@@ -201,7 +204,7 @@ galeon_action_navigation_update (GaleonAction *action)
 			}
 			break;
 		case GALEON_NAVIGATION_DIRECTION_UP:
-			sensitive = (galeon_embed_can_go_up (embed) == G_OK);
+			sensitive = galeon_embed_can_go_up (embed);
 			break;
 		default:
 			sensitive = FALSE;
@@ -261,6 +264,12 @@ galeon_action_navigation_activate (GtkAction *action)
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (embed != NULL);
 
+	if (a->ignore_next_menu_activate)
+	{
+		a->ignore_next_menu_activate = FALSE;
+		return;
+	}
+
 	switch (a->direction)
 	{
 	case GALEON_NAVIGATION_DIRECTION_UP:
@@ -300,15 +309,20 @@ new_num_accel_menu_item (gint num, const gchar *origtext, const gchar *url)
 	if (url)
 	{
 		GaleonFaviconCache *cache = galeon_shell_get_favicon_cache (galeon_shell);
-		GdkPixbuf *icon = galeon_favicon_cache_lookup (cache, url);
-
-		if (icon)
+		GaleonFaviconCacheEntry *entry = galeon_favicon_cache_lookup (cache, url);
+		if (entry)
 		{
-			GtkWidget *image = gtk_image_new_from_pixbuf (icon);
-			
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-						       GTK_WIDGET (image));
-			gtk_widget_show (GTK_WIDGET (image));
+			GdkPixbuf *icon = galeon_favicon_cache_entry_get_pixbuf (entry);
+
+			if (icon)
+			{
+				GtkWidget *image = gtk_image_new_from_pixbuf (icon);
+				
+				gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+							       GTK_WIDGET (image));
+				gtk_widget_show (GTK_WIDGET (image));
+			}
+			g_object_unref (entry);
 		}
 	}
 
@@ -323,6 +337,12 @@ activate_back_or_forward_menu_item_cb (GtkWidget *menu, GaleonActionNavigation *
 	
 	g_return_if_fail (embed != NULL);
 
+	if (a->ignore_next_menu_activate)
+	{
+		a->ignore_next_menu_activate = FALSE;
+		return;
+	}
+
 	galeon_embed_shistory_go_nth (embed, go_nth);
 }
 
@@ -330,24 +350,40 @@ static void
 open_back_or_forward_in_new_tab (GaleonActionNavigation *a, int go_nth)
 {
 	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (a));
-	gchar *url, *title;
 	GaleonWindow *window;
 	GaleonTab *tab;
+	GaleonEmbed *new_embed;
+	GaleonTab *new_tab;
 
 	g_return_if_fail (embed != NULL);
 		
-	if (galeon_embed_shistory_get_nth (embed, go_nth, FALSE, &url, &title) != G_OK)
-	{
-		return;
-	}
-	g_free (title);
-
-	if (!url) return;
-
 	window = galeon_action_get_window (GALEON_ACTION (a));
 	tab = galeon_window_get_active_tab (window);
-	galeon_shell_new_tab (galeon_shell, window, tab, url, GALEON_NEW_TAB_JUMP);
-	g_free (url);
+
+	/* Do some nifty things here, we create a new tab with about:blank
+	 * inside it, and then do a history_go_nth to get the user into the
+	 * correct part of the history navigation */
+	new_tab = galeon_shell_new_tab (galeon_shell, window, tab, NULL,
+					GALEON_NEW_TAB_COPY_HISTORY | 
+					GALEON_NEW_TAB_IS_A_COPY);
+
+	new_embed = galeon_tab_get_embed (new_tab);
+	if (galeon_embed_shistory_go_nth (new_embed, go_nth) == FALSE)
+	{
+		gchar *url, *title;
+		/* If that fails, load the url directly. I can't actually get
+		 * this case to be called, but it is here just for robustness */
+		if (!galeon_embed_shistory_get_nth (embed, go_nth, FALSE, &url, &title))
+		{
+			return;
+		}
+		g_free (title);
+
+		if (!url) return;
+
+		galeon_embed_load_url (new_embed, url);
+		g_free (url);
+	}
 }
 
 static gboolean
@@ -355,10 +391,15 @@ button_press_event_back_or_forward_menu_item_cb  (GtkWidget *widget, GdkEventBut
 						  GaleonActionNavigation *a)
 {
 	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (a));
-	if (event->button == 2 && IS_GALEON_WINDOW (window))
+	if (event->button == 2 && GALEON_IS_WINDOW (window))
 	{
 		int go_nth = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "go_nth"));
 		open_back_or_forward_in_new_tab (a, go_nth);
+
+		/* Ignore the next activation, otherwise we will open this url in the
+		 * current tab as well */
+		a->ignore_next_menu_activate = TRUE;
+
 		return TRUE;
 	}
 	return FALSE;
@@ -374,7 +415,13 @@ activate_up_menu_item_cb (GtkWidget *menu, GaleonActionNavigation *a)
 	
 	g_return_if_fail (embed != NULL);
 
-	galeon_embed_get_go_up_list (embed, &l);
+	if (a->ignore_next_menu_activate)
+	{
+		a->ignore_next_menu_activate = FALSE;
+		return;
+	}
+
+	l = galeon_embed_get_go_up_list (embed);
 
 	url = g_slist_nth_data (l, go_nth);
 	if (url)
@@ -395,14 +442,14 @@ open_up_in_tab (GaleonActionNavigation *a, int go_nth)
 
 	g_return_if_fail (embed != NULL);
 		
-	galeon_embed_get_go_up_list (embed, &l);
+	l = galeon_embed_get_go_up_list (embed);
 	url = g_slist_nth_data (l, go_nth);
 
 	if (url)
 	{
 		GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (a));
 		GaleonTab *tab = galeon_window_get_active_tab (window);
-		galeon_shell_new_tab (galeon_shell, window, tab, url, GALEON_NEW_TAB_JUMP);
+		galeon_shell_new_tab (galeon_shell, window, tab, url, 0);
 	}
 
 	g_slist_foreach (l, (GFunc) g_free, NULL);
@@ -415,10 +462,15 @@ button_press_event_up_menu_item_cb  (GtkWidget *widget, GdkEventButton *event,
 				     GaleonActionNavigation *a)
 {
 	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (a));
-	if (event->button == 2 && IS_GALEON_WINDOW (window))
+	if (event->button == 2 && GALEON_IS_WINDOW (window))
 	{
 		int go_nth = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "go_nth"));
 		open_up_in_tab (a, go_nth);
+
+		/* Ignore the next activation, otherwise we will open this url in the
+		 * current tab as well */
+		a->ignore_next_menu_activate = TRUE;
+
 		return TRUE;
 	}
 	return FALSE;
@@ -432,7 +484,7 @@ galeon_action_navigation_button_press_event_cb (GtkWidget *widget,
 	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (a));
 	GaleonEmbed *embed;
 
-	if (event->button != 2 || ! IS_GALEON_WINDOW (window))
+	if (event->button != 2 || ! GALEON_IS_WINDOW (window))
 	{
 		return FALSE;
 	}
@@ -440,14 +492,23 @@ galeon_action_navigation_button_press_event_cb (GtkWidget *widget,
 	embed = galeon_action_get_embed (GALEON_ACTION (a));
 	g_return_val_if_fail (embed != NULL, FALSE);
 	
+
+	if (GTK_IS_MENU_ITEM (widget))
+	{
+		/* Ignore the next activation, otherwise we will open this url in the
+		 * current tab as well */
+		a->ignore_next_menu_activate = TRUE;
+	}
+
 	if (a->direction == GALEON_NAVIGATION_DIRECTION_UP)
 	{
 		open_up_in_tab (a, 0);
+		return TRUE;
 	}
 	else
 	{
 		int pos;
-		galeon_embed_shistory_get_pos (embed, &pos);
+		pos = galeon_embed_shistory_get_pos (embed);
 		pos = (a->direction == GALEON_NAVIGATION_DIRECTION_BACK) ? pos-1 : pos+1;
 		open_back_or_forward_in_new_tab (a, pos);
 		return TRUE;
@@ -465,8 +526,8 @@ setup_back_or_forward_menu (GaleonActionNavigation *a, GtkMenuShell *ms,
 
 	g_return_if_fail (embed != NULL);
 	
-	galeon_embed_shistory_get_pos (embed, &pos);
-	galeon_embed_shistory_count (embed, &count);
+	pos = galeon_embed_shistory_get_pos (embed);
+	count = galeon_embed_shistory_count (embed);
 
 	if (count == 0) return;
 	
@@ -524,7 +585,7 @@ setup_up_menu (GaleonActionNavigation *a, GtkMenuShell *ms)
 
 	g_return_if_fail (embed != NULL);
 	
-	galeon_embed_get_go_up_list (embed, &l);
+	l = galeon_embed_get_go_up_list (embed);
 	
 	for (li = l; li; li = li->next)
 	{
@@ -598,7 +659,7 @@ galeon_action_navigation_connect_proxy (GtkAction *a, GtkWidget *proxy)
 	g_signal_connect (widget, "button-press-event",
 			  G_CALLBACK (galeon_action_navigation_button_press_event_cb), a);
 
-	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (a, proxy);
+	(* GTK_ACTION_CLASS (galeon_action_navigation_parent_class)->connect_proxy) (a, proxy);
 }
 
 gboolean
