@@ -46,10 +46,6 @@
 #define get_model_from_treeview(treeview) \
 	gtk_tree_model_filter_get_model(get_filter_from_treeview(treeview))
 
-GList *get_model_selection_refs(GtkTreeView *treeview);
-gint tree_row_reference_compare(GtkTreeRowReference *ref1,
-				GtkTreeRowReference *ref2);
-
 typedef struct PdmActionInfo PdmActionInfo;
 
 typedef void (* PDM_add) (PdmActionInfo *info, gpointer data);
@@ -97,10 +93,13 @@ struct PdmActionInfo
 	int remove_id;
 	int data_col;
 	const gchar *filter_string;
-	gboolean lock_selection;
 	PdmDialog *dialog;
 };
 
+#define PDM_DIALOG_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_PDM_DIALOG, PdmDialogPrivate))
+
+
 struct PdmDialogPrivate
 {
 	GtkWidget     *notebook;
@@ -257,7 +256,7 @@ pdm_dialog_get_type (void)
                         (GInstanceInitFunc) pdm_dialog_init
                 };
 
-                pdm_dialog_type = g_type_register_static (GALEON_DIALOG_TYPE,
+                pdm_dialog_type = g_type_register_static (GALEON_TYPE_DIALOG,
 						              "PdmDialog",
 						              &our_info, 0);
         }
@@ -274,57 +273,8 @@ pdm_dialog_class_init (PdmDialogClass *klass)
         parent_class = g_type_class_peek_parent (klass);
 
         object_class->finalize = pdm_dialog_finalize;
-}
 
-GList *get_model_selection_refs(GtkTreeView *treeview)
-{
-	GList *i, *sort_refs, *ret_refs = NULL;
-	GtkTreeModel *model, *sorter, *filter;
-
-	sorter = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
-	filter = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(sorter));
-	model = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER(filter));
-
-	sort_refs = gul_gui_treeview_get_selection_refs(treeview);
-
-	for(i = sort_refs ; i ; i = i->next)
-	{
-		GtkTreePath *sort_path, *filter_path, *path;
-		GtkTreeRowReference *node;
-
-		sort_path = gtk_tree_row_reference_get_path 
-				((GtkTreeRowReference *)i->data);
-		filter_path = gtk_tree_model_sort_convert_path_to_child_path
-				(GTK_TREE_MODEL_SORT(sorter), sort_path);
-		path = gtk_tree_model_filter_convert_path_to_child_path
-				(GTK_TREE_MODEL_FILTER(filter), filter_path);
-
-		node = gtk_tree_row_reference_new(model, path);
-	        ret_refs = g_list_append(ret_refs, node);
-
-		gtk_tree_path_free(path);
-		gtk_tree_path_free(filter_path);
-		gtk_tree_path_free(sort_path);
-	}
-
-	gul_gui_treeview_free_selection_refs(sort_refs);
-
-	return ret_refs;
-}
-
-gint tree_row_reference_compare(GtkTreeRowReference *ref1,
-				GtkTreeRowReference *ref2)
-{
-	gint retval;
-	GtkTreePath *path1 = gtk_tree_row_reference_get_path(ref1);
-	GtkTreePath *path2 = gtk_tree_row_reference_get_path(ref2);
-
-	retval = gtk_tree_path_compare(path1, path2);
-
-	gtk_tree_path_free(path1);
-	gtk_tree_path_free(path2);
-
-	return retval;
+	g_type_class_add_private (klass, sizeof (PdmDialogPrivate));
 }
 
 static void
@@ -343,8 +293,6 @@ cookies_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	GtkWidget *treeview = dialog->priv->cookies->treeview;
 	gchar *url, *scheme;
 
-	if (dialog->priv->cookies->lock_selection) return;
-
 	value_entry = GTK_ENTRY(galeon_dialog_get_control(d, PROP_COOKIES_VALUE_ENTRY));
 	url_entry = GTK_ENTRY(galeon_dialog_get_control(d, PROP_COOKIES_URL_ENTRY));
 	expires_label = GTK_LABEL(galeon_dialog_get_control(d, PROP_COOKIES_EXPIRES_LABEL));
@@ -374,7 +322,7 @@ cookies_treeview_selection_changed_cb (GtkTreeSelection *selection,
 		(model, &iter, COL_COOKIES_DATA, &val);
 	info = (CookieInfo *)g_value_get_pointer (&val);
 
-	if (!g_ascii_strcasecmp(info->secure, "Yes"))
+	if (info->secure)
 		scheme = "https://";
 	else
 		scheme = "http://";
@@ -399,8 +347,6 @@ action_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	GaleonDialog *d = GALEON_DIALOG(action->dialog);
 	gboolean has_selection;
 
-	if (action->lock_selection) return;
-
 	l = gul_gui_treeview_get_selection_refs 
 		(GTK_TREE_VIEW(action->treeview));
 
@@ -494,11 +440,15 @@ setup_passwords_treeview (PdmDialog *dialog)
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
                                         G_TYPE_POINTER);
-
 	filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(liststore), NULL);
 	sorter = gtk_tree_model_sort_new_with_model(filter);
 
         gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(sorter));
+
+	g_object_unref (liststore);
+	g_object_unref (filter);
+	g_object_unref (sorter);
+
         gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection,
@@ -553,11 +503,15 @@ setup_password_sites_treeview (PdmDialog *dialog)
                                        G_TYPE_STRING,
                                        G_TYPE_STRING,
                                        G_TYPE_POINTER);
-
 	filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(liststore), NULL);
 	sorter = gtk_tree_model_sort_new_with_model(filter);
 
         gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(sorter));
+
+	g_object_unref (liststore);
+	g_object_unref (filter);
+	g_object_unref (sorter);
+
         gtk_tree_view_set_headers_visible(treeview, TRUE);
 	selection = gtk_tree_view_get_selection(treeview);
 	gtk_tree_selection_set_mode(selection,
@@ -611,11 +565,15 @@ setup_cookies_treeview (PdmDialog *dialog)
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
                                         G_TYPE_POINTER);
-
 	filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(liststore), NULL);
 	sorter = gtk_tree_model_sort_new_with_model(filter);
 
         gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(sorter));
+
+	g_object_unref (liststore);
+	g_object_unref (filter);
+	g_object_unref (sorter);
+
         gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection,
@@ -675,11 +633,15 @@ setup_cookie_sites_treeview (PdmDialog *dialog)
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
                                         G_TYPE_POINTER);
-
 	filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(liststore), NULL);
 	sorter = gtk_tree_model_sort_new_with_model(filter);
 
         gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(sorter));
+
+	g_object_unref (liststore);
+	g_object_unref (filter);
+	g_object_unref (sorter);
+
         gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection,
@@ -735,11 +697,15 @@ setup_image_sites_treeview (PdmDialog *dialog)
                                        G_TYPE_STRING,
                                        G_TYPE_STRING,
                                        G_TYPE_POINTER);
-
 	filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(liststore), NULL);
 	sorter = gtk_tree_model_sort_new_with_model(filter);
 
         gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(sorter));
+
+	g_object_unref (liststore);
+	g_object_unref (filter);
+	g_object_unref (sorter);
+
         gtk_tree_view_set_headers_visible(treeview, TRUE);
 	selection = gtk_tree_view_get_selection(treeview);
 	gtk_tree_selection_set_mode(selection,
@@ -795,11 +761,15 @@ setup_popup_sites_treeview (PdmDialog *dialog)
                                        G_TYPE_STRING,
                                        G_TYPE_STRING,
                                        G_TYPE_POINTER);
-
 	filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(liststore), NULL);
 	sorter = gtk_tree_model_sort_new_with_model(filter);
 
         gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(sorter));
+
+	g_object_unref (liststore);
+	g_object_unref (filter);
+	g_object_unref (sorter);
+
         gtk_tree_view_set_headers_visible(treeview, TRUE);
 	selection = gtk_tree_view_get_selection(treeview);
 	gtk_tree_selection_set_mode(selection,
@@ -837,59 +807,79 @@ setup_popup_sites_treeview (PdmDialog *dialog)
 static GList *
 get_remove_list(PdmActionInfo *action)
 {
-	GList *refs, *l;
-	GList *remove_list = NULL;
+	GList *l, *model_iters, *sorter_paths, *remove_list;
 	GtkTreeModel *model, *sorter, *filter;
+	GtkTreeSelection *selection;
 
-	sorter = gtk_tree_view_get_model(GTK_TREE_VIEW(action->treeview));
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(action->treeview));
+	sorter_paths = gtk_tree_selection_get_selected_rows(selection, &sorter);
+
+	/* We have our list of selected paths, update selection now to avoid
+	 * multiple signals when we start removing things.  If only single row
+	 * was selected, attempt to select the next one below.  No harm done if
+	 * it fails (we're not trying to select the row above because Evolution
+	 * isn't either.)
+	 */
+	gtk_tree_selection_unselect_all(selection);
+	if (sorter_paths != NULL && sorter_paths->next == NULL)
+	{
+		GtkTreePath *select_path;
+
+		select_path = gtk_tree_path_copy(sorter_paths->data);
+		gtk_tree_path_next(select_path);
+		gtk_tree_selection_select_path(selection, select_path);
+		gtk_tree_path_free(select_path);
+	}
+
+	/* Paths are somewhat fragile, removing a row will invalidate all the
+	 * following paths (they will be referring to the row below the one
+	 * they used to) and you'll end up removing random items.  Convert
+	 * paths to TreeIters which persist (at least with ListStore they do.)
+	 */
 	filter = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(sorter));
 	model = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER(filter));
 
-	action->lock_selection = TRUE;
-
-	refs = get_model_selection_refs(GTK_TREE_VIEW(action->treeview));
-	refs = g_list_sort(refs, (GCompareFunc)tree_row_reference_compare);
-	refs = g_list_reverse(refs);
-
-	for (l = refs; l != NULL; l = l->next)
+	model_iters = NULL;
+	for (l = sorter_paths; l != NULL; l = l->next)
 	{
-		GtkTreeIter iter;
-		gpointer data;
-		GtkTreePath *path;
-		GValue val = {0, };
+		GtkTreePath *sorter_path = l->data;
+		GtkTreePath *filter_path, *model_path;
+		GtkTreeIter  iter;
 
-		if (!gtk_tree_row_reference_valid((GtkTreeRowReference *)l->data))
+		filter_path = gtk_tree_model_sort_convert_path_to_child_path
+			(GTK_TREE_MODEL_SORT(sorter), sorter_path);
+		model_path = gtk_tree_model_filter_convert_path_to_child_path
+			(GTK_TREE_MODEL_FILTER(filter), filter_path);
+
+		if (gtk_tree_model_get_iter(model, &iter, model_path))
 		{
-			continue;
+			model_iters = g_list_prepend(model_iters, gtk_tree_iter_copy(&iter));
 		}
 
-		path = gtk_tree_row_reference_get_path 
-				((GtkTreeRowReference *)l->data);
-
-		gtk_tree_model_get_iter(model, &iter, path);
-
-		gtk_tree_model_get_value 
-			(model, &iter, action->data_col, &val);
-		data = g_value_get_pointer (&val);
-
-		gtk_list_store_remove (GTK_LIST_STORE(model),
-				       &iter);
-
-		gtk_tree_model_row_deleted(model, path);
-		gtk_tree_path_free(path);
-
-		action->list = g_list_remove (action->list, data);
-		remove_list = g_list_append (remove_list, data);
+		gtk_tree_path_free(model_path);
+		gtk_tree_path_free(filter_path);
+		gtk_tree_path_free(sorter_path);
 	}
+	g_list_free(sorter_paths);
 
-	gul_gui_treeview_free_selection_refs(refs);
+	/* It's now safe to turn off your computer... I mean, remove the rows. */
+	remove_list = NULL;
+	for (l = model_iters; l != NULL; l = l->next)
+	{
+		GtkTreeIter *iter = l->data;
+		GList       *link;
+		gpointer     data;
 
-	gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(filter));
+		gtk_tree_model_get(model, iter, action->data_col, &data, -1);
 
-	action->lock_selection = FALSE;
+		link = g_list_find(action->list, data);
+		action->list = g_list_remove_link(action->list, link);
+		remove_list = g_list_concat(link, remove_list);
 
-	/* Callback will clear the cookie info fields */
-	cookies_treeview_selection_changed_cb(NULL, action->dialog);
+		gtk_list_store_remove(GTK_LIST_STORE(model), iter);
+		gtk_tree_iter_free(iter);
+	}
+	g_list_free(model_iters);
 
 	return remove_list;
 }
@@ -1019,8 +1009,7 @@ pdm_dialog_cookie_sites_reload_cb(GtkWidget *button,
 
 	shell = galeon_shell_get_embed_shell(galeon_shell);
 
-	galeon_embed_shell_list_permissions(shell, COOKIES_PERMISSION,
-					    &action->list);
+	action->list = galeon_embed_shell_list_permissions (shell, COOKIES_PERMISSION);
 
 	for (l = action->list; l != NULL; l = l->next)
 	{
@@ -1250,7 +1239,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	
 	shell = galeon_shell_get_embed_shell (galeon_shell);
 	
-	dialog->priv = g_new0 (PdmDialogPrivate, 1);
+	dialog->priv = PDM_DIALOG_GET_PRIVATE (dialog);
 	dialog->priv->cookies = NULL;
 	dialog->priv->passwords = NULL;
 	
@@ -1277,7 +1266,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	popup_sites_tv = setup_popup_sites_treeview(dialog);
 
 	cookies = g_new0 (PdmActionInfo, 1);
-	galeon_embed_shell_list_cookies (shell, &cookies->list);
+	cookies->list = galeon_embed_shell_list_cookies (shell);
 	cookies->dialog = dialog;
 	cookies->remove_id = PROP_COOKIES_REMOVE;
 	cookies->add = pdm_dialog_cookie_add;
@@ -1296,8 +1285,7 @@ pdm_dialog_init (PdmDialog *dialog)
 			 cookies);
 
 	passwords = g_new0 (PdmActionInfo, 1);
-	galeon_embed_shell_list_passwords (shell, PASSWORD_PASSWORD,
-					   &passwords->list);
+	passwords->list = galeon_embed_shell_list_passwords (shell, PASSWORD_PASSWORD);
 	passwords->dialog = dialog;
 	passwords->remove_id = PROP_PASSWORDS_REMOVE;
 	passwords->add = pdm_dialog_password_add;
@@ -1308,8 +1296,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	setup_action (passwords);
 
 	password_sites = g_new0(PdmActionInfo, 1);
-	galeon_embed_shell_list_passwords(shell, PASSWORD_REJECT,
-					  &password_sites->list);
+	password_sites->list = galeon_embed_shell_list_passwords(shell, PASSWORD_REJECT);
 	password_sites->dialog = dialog;
 	password_sites->remove_id = PROP_PASSWORD_SITES_REMOVE;
 	password_sites->add = pdm_dialog_password_site_add;
@@ -1320,8 +1307,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	setup_action(password_sites);
 
 	cookie_sites = g_new0 (PdmActionInfo, 1);
-	galeon_embed_shell_list_permissions (shell, COOKIES_PERMISSION,
-					     &cookie_sites->list);
+	cookie_sites->list = galeon_embed_shell_list_permissions (shell, COOKIES_PERMISSION);
 	cookie_sites->dialog	= dialog;
 	cookie_sites->remove_id = PROP_COOKIE_SITES_REMOVE;
 	cookie_sites->add	= pdm_dialog_cookie_site_add;
@@ -1340,8 +1326,7 @@ pdm_dialog_init (PdmDialog *dialog)
 			 cookie_sites);
 
 	image_sites = g_new0(PdmActionInfo, 1);
-	galeon_embed_shell_list_permissions(shell, IMAGES_PERMISSION,
-					    &image_sites->list);
+	image_sites->list = galeon_embed_shell_list_permissions(shell, IMAGES_PERMISSION);
 	image_sites->dialog	= dialog;
 	image_sites->remove_id	= PROP_IMAGE_SITES_REMOVE;
 	image_sites->add	= pdm_dialog_image_site_add;
@@ -1352,8 +1337,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	setup_action(image_sites);
 
 	popup_sites = g_new0(PdmActionInfo, 1);
-	galeon_embed_shell_list_permissions(shell, POPUPS_PERMISSION,
-					    &popup_sites->list);
+	popup_sites->list = galeon_embed_shell_list_permissions(shell, POPUPS_PERMISSION);
 	popup_sites->dialog	= dialog;
 	popup_sites->remove_id	= PROP_POPUP_SITES_REMOVE;
 	popup_sites->add	= pdm_dialog_popup_site_add;
@@ -1409,7 +1393,6 @@ pdm_dialog_finalize (GObject *object)
 	pdm_dialog_passwords_free (dialog->priv->password_sites, NULL);
 	pdm_dialog_cookies_free (dialog->priv->cookies, NULL);
 	
-        g_free (dialog->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -1419,7 +1402,7 @@ pdm_dialog_new (GtkWidget *window)
 {
 	PdmDialog *dialog;
 	
-	dialog = PDM_DIALOG (g_object_new (PDM_DIALOG_TYPE,
+	dialog = PDM_DIALOG (g_object_new (TYPE_PDM_DIALOG,
 				     	   "ParentWindow", window,
 				           NULL));
 
