@@ -26,10 +26,17 @@
 #include <string.h>
 #include <unistd.h>
 #include <glib.h>
-#include <libgnome/libgnome.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include <libgnome/gnome-exec.h>
 #include <libxml/xmlmemory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+/* http://bugzilla.gnome.org/show_bug.cgi?id=156687 */
+# undef GNOME_DISABLE_DEPRECATED
+# include <libgnome/gnome-desktop-item.h>
+#endif
 
 char *
 gul_general_tmp_filename (const char *base,
@@ -38,7 +45,7 @@ gul_general_tmp_filename (const char *base,
 	int fd;
 	char *name = g_strdup (base);
 	
-	fd = mkstemp (name);
+	fd = g_mkstemp (name);
 
 	if (fd != -1)
     	{
@@ -62,6 +69,85 @@ gul_general_tmp_filename (const char *base,
 	return name;
 }
 
+
+/**
+ * gul_general_switch_temp_file: Perform a safe switch of a tmp file
+ * the the real filename
+ */
+gboolean
+gul_general_switch_temp_file (const char *filename,
+			      const char *filename_temp)
+{
+	char *old_file;
+	gboolean old_exist;
+	gboolean retval = TRUE;
+
+	old_file = g_strconcat (filename, ".old", NULL);
+
+	old_exist = g_file_test (filename, G_FILE_TEST_EXISTS);
+
+	if (old_exist)
+	{
+		if (rename (filename, old_file) < 0)
+		{
+			g_warning ("Failed to rename %s to %s", filename, old_file);
+			retval = FALSE;
+			goto failed;
+		}
+	}
+
+	if (rename (filename_temp, filename) < 0)
+	{
+		g_warning ("Failed to rename %s to %s", filename_temp, filename);
+
+		if (rename (old_file, filename) < 0)
+		{
+			g_warning ("Failed to restore %s from %s",
+				   filename, filename_temp);
+		}
+		retval = FALSE;
+		goto failed;
+	}
+
+	if (old_exist)
+	{
+		if (unlink (old_file) < 0)
+		{
+			g_warning ("Failed to delete old file %s", old_file);
+		}
+	}
+
+failed:
+	g_free (old_file);
+
+	return retval;
+}
+
+
+/**
+ * gul_general_safe_xml_save: Saves an XMLdoc in a low-disk-safe manner
+ */
+gboolean
+gul_general_safe_xml_save    (const char *filename, xmlDocPtr doc)
+{
+	char *tmpfile = g_strconcat (filename, ".tmp", NULL);
+	int ret = xmlSaveFormatFile (tmpfile, doc, 1);
+
+	if (ret < 0)
+	{
+		g_warning ("Failed to save %s", filename);
+	}
+	else
+	{
+		gul_general_switch_temp_file (filename, tmpfile);
+	}
+
+	g_free (tmpfile);
+
+	return ( ret < 0 ) ? FALSE : TRUE;
+}
+
+
 /**
  * misc_general_user_file: returns the pathname of galeon shared files
  * (e.g., galeon.glade)
@@ -80,14 +166,22 @@ gul_general_user_file (const char *fname, gboolean critical)
 	/* create cache hash table if it doesnt already exist */
 	if (already_found == NULL)
 	{
-		already_found = g_hash_table_new (g_str_hash, g_str_equal);
+		already_found = g_hash_table_new_full (g_str_hash, g_str_equal,
+						       g_free, g_free);
 	}
 
         /* Have we already found this? */
 	file = g_hash_table_lookup (already_found, fname);
-	if (file != NULL) 
+	if (file != NULL)
 	{
-		return g_strdup (file);
+		if (g_file_test (file, G_FILE_TEST_EXISTS))
+		{
+			return g_strdup (file);
+		}
+		else
+		{
+			g_hash_table_remove (already_found, fname);
+		}
 	}
 
 	/* try the default */
@@ -136,7 +230,15 @@ gul_general_user_file (const char *fname, gboolean critical)
 		/* warn if we're using other than the install default */
 		if (!g_path_is_absolute (file))
 		{
+			gchar * cwd, *file1;
 			g_message ("Using %s (usually OK)", file);
+
+			/* Make it absolute */
+			cwd = g_get_current_dir();
+			file1 = g_build_filename (cwd, file, NULL);
+			g_free (file);
+			g_free (cwd);
+			file = file1;
 		}
 
 		/* add it to the set of found files */
@@ -153,59 +255,6 @@ gul_general_user_file (const char *fname, gboolean critical)
 	return file;
 }
 
-static gchar *
-gul_string_escape_xml_prop (const gchar *text)
-{
-        GString *out = g_string_new (NULL);
-        guchar c;
-        while ((c = *text))
-        {
-                if (c<0x20 || c>0x7F || c=='&' || c=='<' || c=='>' || c=='%' ||
-                                c=='"') //Should anything else be escaped?
-                        g_string_append_printf (out, "%%%02X", c);
-                else
-                        g_string_append_c (out, c);
-                ++text;
-        }
-        text = out->str;
-        g_string_free (out, FALSE); /* don't free char data, caller must */
-        return (gchar *)text;
-}
-
-/**
- * xmlSetRawProp: set a property in an XML file which has
- * value encoded.  This works around the newline-losingness and broken
- * entitiy handling of libxml1. (see bug #56487)
- */
-xmlAttrPtr
-xmlSetRawProp (xmlNodePtr node, const xmlChar *name, const xmlChar *value)
-{
-	xmlAttrPtr attr;
-	gchar *escaped = gul_string_escape_xml_prop (value);
-	attr = xmlSetProp (node, name, escaped);
-	g_free (escaped);
-	return attr;
-}
-/**
- * xmlGetRawProp: get a property in an XML file which has been encoded This
- * works around the newline-losingness and broken entitiy handling of
- * libxml1. (see bug #56487)
- */
-xmlChar *
-xmlGetRawProp (xmlNodePtr node, const xmlChar *name)
-{
-	xmlChar *value;
-	gchar *unescaped;
-	value = xmlGetProp (node, name);
-	if (value == NULL)
-	{
-		return NULL;
-	}
-	unescaped = gul_string_unescape_hexed_string (value);
-	xmlFree (value);
-	return unescaped;
-}
-
 /**
  * xmlGetIntProp:
  */
@@ -328,180 +377,49 @@ gul_general_read_line_from_file (FILE *f)
 	return line;
 }
 
-static void 
-gul_general_gnome_shell_execute (const char *command)
+GnomeVFSResult
+gul_general_launch_application (GnomeVFSMimeApplication *application,
+				const char              *parameter,
+				guint32 user_time)
 {
-        GError *error = NULL;
-        if (!g_spawn_command_line_async (command, &error)) {
-                g_warning ("Error starting command '%s': %s\n", command, error->message);
-                g_error_free (error);
-        }
-}
+	GList         *uris;
+	GnomeVFSResult rv;
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+	const char *desktop_file;
+	GnomeDesktopItem *desktop_item;
+	int ret;
+#endif
+	g_return_val_if_fail (application != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (parameter != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
-/* Return a command string containing the path to a terminal on this system. */
+	uris = g_list_prepend (NULL, gnome_vfs_make_uri_canonical (parameter));
 
-static char *
-try_terminal_command (const char *program,
-                      const char *args)
-{
-        char *program_in_path, *quoted, *result;
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+	desktop_file = gnome_vfs_mime_application_get_desktop_file_path (application);
+	desktop_item = gnome_desktop_item_new_from_file (desktop_file,
+							 GNOME_DESKTOP_ITEM_LOAD_NO_TRANSLATIONS,
+							 NULL);
+	if (!desktop_item)
+	{
+		return GNOME_VFS_ERROR_INTERNAL;
+	}
 
-        if (program == NULL) {
-                return NULL;
-        }
+	gnome_desktop_item_set_launch_time (desktop_item, user_time);
+	ret = gnome_desktop_item_launch (desktop_item, uris,
+					 GNOME_DESKTOP_ITEM_LAUNCH_APPEND_PATHS, NULL);
 
-        program_in_path = g_find_program_in_path (program);
-        if (program_in_path == NULL) {
-                return NULL;
-        }
+	gnome_desktop_item_unref (desktop_item);
 
-        quoted = g_shell_quote (program_in_path);
-        if (args == NULL || args[0] == '\0') {
-                return quoted;
-        }
-        result = g_strconcat (quoted, " ", args, NULL);
-        g_free (quoted);
-        return result;
-}
+	return ret == -1 ? GNOME_VFS_OK : GNOME_VFS_ERROR_GENERIC;
+#else
 
-static char *
-try_terminal_command_argv (int argc,
-                           char **argv)
-{
-        GString *string;
-        int i;
-        char *quoted, *result;
+	rv = gnome_vfs_mime_application_launch (application, uris);
 
-        if (argc == 0) {
-                return NULL;
-        }
+	g_list_foreach (uris, (GFunc)g_free, NULL);
+	g_list_free (uris);
+#endif
 
-        if (argc == 1) {
-                return try_terminal_command (argv[0], NULL);
-        }
-        
-        string = g_string_new (argv[1]);
-        for (i = 2; i < argc; i++) {
-                quoted = g_shell_quote (argv[i]);
-                g_string_append_c (string, ' ');
-                g_string_append (string, quoted);
-                g_free (quoted);
-        }
-        result = try_terminal_command (argv[0], string->str);
-        g_string_free (string, TRUE);
-
-        return result;
-}
-
-static char *
-get_terminal_command_prefix (gboolean for_command)
-{
-        int argc;
-        char **argv;
-        char *command;
-        guint i;
-        static const char *const commands[][3] = {
-                { "gnome-terminal", "-x",                                      "" },
-                { "dtterm",         "-e",                                      "-ls" },
-                { "nxterm",         "-e",                                      "-ls" },
-                { "color-xterm",    "-e",                                      "-ls" },
-                { "rxvt",           "-e",                                      "-ls" },
-                { "xterm",          "-e",                                      "-ls" },
-        };
-
-        /* Try the terminal from preferences. Use without any
-         * arguments if we are just doing a standalone terminal.
-         */
-        argc = 0;
-        argv = g_new0 (char *, 1);
-        gnome_prepend_terminal_to_vector (&argc, &argv);
-
-        command = NULL;
-        if (argc != 0) {
-                if (for_command) {
-                        command = try_terminal_command_argv (argc, argv);
-                } else {
-                        /* Strip off the arguments in a lame attempt
-                         * to make it be an interactive shell.
-                         */
-                        command = try_terminal_command (argv[0], NULL);
-                }
-        }
-
-        while (argc != 0) {
-                g_free (argv[--argc]);
-        }
-        g_free (argv);
-
-        if (command != NULL) {
-                return command;
-        }
-
-        /* Try well-known terminal applications in same order that gmc did. */
-        for (i = 0; i < G_N_ELEMENTS (commands); i++) {
-                command = try_terminal_command (commands[i][0],
-                                                commands[i][for_command ? 1 : 2]);
-                if (command != NULL) {
-                        break;
-                }
-        }
-        
-        return command;
-}
-
-static char *
-gul_general_gnome_make_terminal_command (const char *command)
-{
-        char *prefix, *quoted, *terminal_command;
-
-        if (command == NULL) {
-                return get_terminal_command_prefix (FALSE);
-        }
-        prefix = get_terminal_command_prefix (TRUE);
-        quoted = g_shell_quote (command);
-        terminal_command = g_strconcat (prefix, " /bin/sh -c ", quoted, NULL);
-        g_free (prefix);
-        g_free (quoted);
-        return terminal_command;
-}
-
-static void
-gul_general_gnome_open_terminal (const char *command)
-{
-        char *command_line;
-        
-        command_line = gul_general_gnome_make_terminal_command (command);
-        if (command_line == NULL) {
-                g_message ("Could not start a terminal");
-                return;
-        }
-        gul_general_gnome_shell_execute (command_line);
-        g_free (command_line);
-}
-
-void
-gul_general_launch_application (const char *command_string, 
-                                const char *parameter, 
-                                gboolean use_terminal)
-{
-        char *full_command;
-        char *quoted_parameter; 
-
-        if (parameter != NULL) {
-                quoted_parameter = g_shell_quote (parameter);
-                full_command = g_strconcat (command_string, " ", quoted_parameter, NULL);
-                g_free (quoted_parameter);
-        } else {
-                full_command = g_strdup (command_string);
-        }
-        
-        if (use_terminal) {
-                gul_general_gnome_open_terminal (full_command);
-        } else {
-                gul_general_gnome_shell_execute (full_command);
-        }
-
-        g_free (full_command);
+	return rv;
 }
 
 static void
@@ -598,3 +516,20 @@ gul_copy_file (const char *orig,
 	return TRUE;
 }
 
+GSList *
+gul_slist_filter (const GSList *l, GulFilterFunc f, gpointer data)
+{
+	GSList *ret = NULL;
+	const GSList *li;
+
+	for (li = l; li; li = li->next)
+	{
+		if (f (li->data, data))
+		{
+			ret = g_slist_prepend (ret, li->data);
+		}
+	}
+
+	return g_slist_reverse (ret);
+}
+
