@@ -28,29 +28,44 @@
 
 #include "gul-notebook.h"
 #include "gul-gui.h"
+#include "gul-string.h"
 #include "galeon-marshal.h"
 
-#include <gtk/gtk.h>
-#include <glib-object.h>
-#include <libgnome/gnome-i18n.h>
+#include <gtk/gtkalignment.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkeventbox.h>
 
-#define AFTER_ALL_TABS -1
-#define NOT_IN_APP_WINDOWS -2
-#define TAB_MIN_SIZE 60
-#define TAB_NB_MAX    8
+#include <glib-object.h>
+#include <glib/gi18n.h>
+
+#define AFTER_ALL_TABS          -1
+#define NOT_IN_APP_WINDOWS      -2
+
+#define DEFAULT_TAB_WIDTH_CHARS 15
+#define MENU_ITEM_MAX_LENGTH    40
+
+/* spacing between tab border and favicon/label, and between label and close
+ * button */
+#define SPACING              3
 
 struct GulNotebookPrivate 
 {
 	GList *focused_pages;
 	GList *opened_tabs;
 	GtkPolicyType policy;
+	gboolean automatic_tab_switch;
+	gint tab_width_chars;
 	
 	/* Used during tab drag'n'drop */
 	gulong motion_notify_handler_id;
 	gint x_start, y_start;
 	gboolean drag_in_progress;
-	GulNotebook *src_notebook;
-	gint src_page;
 };
 
 /* GObject boilerplate code */
@@ -61,12 +76,11 @@ static void gul_notebook_finalize     (GObject *object);
 /* Local variables */
 static GdkCursor *cursor = NULL;
 static GList *notebooks  = NULL;
+static GtkTooltips *_gul_notebook_tooltips = NULL;
 
 
 /* Local functions */
-static void drag_start (GulNotebook *notebook, 
-			GulNotebook *src_notebook,
-			gint src_page);
+static void drag_start (GulNotebook *notebook);
 static void drag_stop  (GulNotebook *notebook);
 
 static gboolean motion_notify_cb (GulNotebook *notebook, 
@@ -76,12 +90,16 @@ static gboolean motion_notify_cb (GulNotebook *notebook,
 /* Signals */
 enum
 {
-	TAB_DROPPED,
+	TAB_ADDED,
+	TAB_REMOVED,
+	TABS_REORDERED,
 	TAB_DETACHED,
+	TAB_DELETE,
 	LAST_SIGNAL
 };
 
 static guint gul_notebook_signals[LAST_SIGNAL] = { 0 };
+static GObjectClass *parent_class = NULL;
 
 GType 
 gul_notebook_get_type (void)
@@ -115,22 +133,31 @@ static void
 gul_notebook_class_init (GulNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	parent_class = g_type_class_peek_parent (klass);
+
 	object_class->finalize = gul_notebook_finalize;
 
 	/* init signals */
-	gul_notebook_signals[TAB_DROPPED] =
-		g_signal_new ("tab_dropped",
+	gul_notebook_signals[TAB_ADDED] =
+		g_signal_new ("tab_added",
 			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GulNotebookClass, 
-					       tab_dropped),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tab_added),
 			      NULL, NULL,
-			      galeon_marshal_VOID__OBJECT_OBJECT_INT,
+			      galeon_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      3, 
-			      GTK_TYPE_WIDGET,
-			      GUL_NOTEBOOK_TYPE,
-			      G_TYPE_INT);
+			      1,
+			      GTK_TYPE_WIDGET);
+	gul_notebook_signals[TAB_REMOVED] =
+		g_signal_new ("tab_removed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tab_removed),
+			      NULL, NULL,
+			      galeon_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      GTK_TYPE_WIDGET);
 	gul_notebook_signals[TAB_DETACHED] =
 		g_signal_new ("tab_detached",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -144,7 +171,26 @@ gul_notebook_class_init (GulNotebookClass *klass)
 			      G_TYPE_INT,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
+	gul_notebook_signals[TABS_REORDERED] =
+		g_signal_new ("tabs_reordered",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tabs_reordered),
+			      NULL, NULL,
+			      galeon_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 
+	gul_notebook_signals[TAB_DELETE] =
+		g_signal_new ("tab_delete",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tab_delete),
+			      NULL, NULL,
+			      galeon_marshal_BOOLEAN__OBJECT,
+			      G_TYPE_BOOLEAN,
+			      1,
+			      GTK_TYPE_WIDGET);
 }
 
 static gboolean 
@@ -174,7 +220,7 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 	GList *l;
 	gint x, y;
 	GdkWindow *win_at_pointer = gdk_window_at_pointer (&x, &y);
-	GdkWindow *parent_at_pointer = NULL;
+	GdkWindow *toplevel_at_pointer = NULL;
 
 	if (win_at_pointer == NULL) 
 	{
@@ -182,12 +228,8 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 		return NULL;
 	}
 
-	gdk_window_get_toplevel (win_at_pointer);
-	/* When we are in the notebook event window, win_at_pointer will be
-	   this event window, and the toplevel window we are interested in
-	   will be its parent
-	*/
-	parent_at_pointer = gdk_window_get_parent (win_at_pointer);
+	/* Get the toplevel window for where the pointer is currently */
+	toplevel_at_pointer = gdk_window_get_toplevel (win_at_pointer);
 
 	for (l = notebooks; l != NULL; l = l->next) 
 	{
@@ -195,7 +237,7 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 		GdkWindow *win = GTK_WIDGET (nb)->window;
 
 		win = gdk_window_get_toplevel (win);
-		if (((win == win_at_pointer) || (win == parent_at_pointer))
+		if (win == toplevel_at_pointer
 		    && is_in_notebook_window (nb, abs_x, abs_y)) 
 		{
 			return nb;
@@ -283,18 +325,6 @@ static gint find_notebook_and_tab_at_pos (gint abs_x, gint abs_y,
 	}
 }
 
-static void
-tab_label_set_size (GtkWidget *window, GtkWidget *label)
-{
-	int label_width;
-	
-	label_width = window->allocation.width/TAB_NB_MAX;
-
-	if (label_width < TAB_MIN_SIZE) label_width = TAB_MIN_SIZE;
-	
-	gtk_widget_set_size_request (label, label_width, -1);
-}
-
 static GtkWidget *
 tab_get_label (GulNotebook *nb, GtkWidget *child)
 {
@@ -307,44 +337,59 @@ tab_get_label (GulNotebook *nb, GtkWidget *child)
 	return label;
 }
 
-static void
-tab_label_size_request_cb (GtkWidget *window,
-			   GtkRequisition *requisition,
-			   GtkWidget *child)
-{
-	GtkWidget *hbox;
-	GtkWidget *nb;
 
-	nb = child->parent;
-	
+static GtkWidget *
+tab_get_icon (GulNotebook *nb, GtkWidget *child)
+{
+	GtkWidget *hbox, *icon;
+
 	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb),
 					   child);
-	tab_label_set_size (window, hbox);
-}
+	icon = g_object_get_data (G_OBJECT (hbox), "icon");
 
+	return icon;
+}
 
 void
 gul_notebook_move_page (GulNotebook *src, GulNotebook *dest, 
 			GtkWidget *src_page,  gint dest_page)
 {
-	GtkWidget *tab_label;
-	GtkWidget *menu_label;
+	GtkWidget *tab_label, *icon;
+	const gchar * text;
 
-	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (src), src_page);
-	menu_label = gtk_notebook_get_menu_label (GTK_NOTEBOOK (src), src_page);
+	if (dest == NULL || src == dest)
+	{
+		gtk_notebook_reorder_child (GTK_NOTEBOOK (src), src_page, dest_page);
+		
+		if (src->priv->drag_in_progress == FALSE)
+		{
+			g_signal_emit (G_OBJECT (src), 
+				       gul_notebook_signals[TABS_REORDERED], 0);
+		}
+		return;
+	}
+
+	tab_label = tab_get_label (src, src_page);
+	icon      = tab_get_icon  (src, src_page);
 
 	/* We don't want gtk to destroy tab and src_page behind our back */
 	g_object_ref (G_OBJECT (src_page));
 	g_object_ref (G_OBJECT (tab_label));
-	g_object_ref (G_OBJECT (menu_label));
+	if (icon) g_object_ref (G_OBJECT (icon));
+
 	gul_notebook_remove_page (GUL_NOTEBOOK (src), src_page);
 	gul_notebook_insert_page (GUL_NOTEBOOK (dest), src_page,
 				  dest_page, TRUE);
-	gtk_notebook_set_tab_label (GTK_NOTEBOOK (dest), src_page, tab_label);
-	gtk_notebook_set_menu_label (GTK_NOTEBOOK (dest), src_page, menu_label);
+	/* Restore the text  */
+	text = gtk_label_get_text( GTK_LABEL(tab_label) );
+	gul_notebook_set_page_title( dest, src_page, text );
+	if (icon)
+	{
+		gul_notebook_set_page_icon (dest, src_page, icon);
+		g_object_unref (G_OBJECT (icon));
+	}
 	g_object_unref (G_OBJECT (src_page));
 	g_object_unref (G_OBJECT (tab_label));
-	g_object_unref (G_OBJECT (menu_label));
 }
 
 
@@ -372,7 +417,7 @@ move_tab_to_another_notebook(GulNotebook *src,
 	gul_notebook_move_page (src, dest, child, dest_page);
 
 	/* "Give" drag handling to the new notebook */
-	drag_start (dest, src->priv->src_notebook, src->priv->src_page);
+	drag_start (dest);
 	drag_stop (src);		
 	gtk_grab_remove (GTK_WIDGET (src));
 	
@@ -406,13 +451,9 @@ move_tab (GulNotebook *notebook, gint dest_page_num)
 }
 
 static void 
-drag_start (GulNotebook *notebook, 
-	    GulNotebook *src_notebook, 
-	    gint src_page)
+drag_start (GulNotebook *notebook)
 {
 	notebook->priv->drag_in_progress = TRUE;
-	notebook->priv->src_notebook = src_notebook;
-	notebook->priv->src_page = src_page;
 
 	/* get a new cursor, if necessary */
 	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
@@ -430,9 +471,14 @@ drag_start (GulNotebook *notebook,
 static void 
 drag_stop (GulNotebook *notebook)
 {
+	if (notebook->priv->drag_in_progress)
+	{
+		g_signal_emit (G_OBJECT (notebook), 
+			       gul_notebook_signals[TABS_REORDERED], 0);
+	}
+
 	notebook->priv->drag_in_progress = FALSE;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
+
 	if (notebook->priv->motion_notify_handler_id != 0) 
 	{
 		g_signal_handler_disconnect (G_OBJECT (notebook), 
@@ -464,16 +510,6 @@ button_release_cb (GulNotebook *notebook, GdkEventButton *event,
 				       cur_page_num, (gint)event->x_root, 
 				       (gint)event->y_root);
 		}
-		else
-		{
-			/* Tab was dragged and dropped (but it may have stayed
-			   in the same place) */
-			g_signal_emit (G_OBJECT(notebook), 
-				       gul_notebook_signals[TAB_DROPPED], 0,
-				       cur_page, 
-				       notebook->priv->src_notebook, 
-				       notebook->priv->src_page);
-		}
 
 		/* ungrab the pointer if it's grabbed */
 		if (gdk_pointer_is_grabbed ())
@@ -503,16 +539,19 @@ motion_notify_cb (GulNotebook *notebook, GdkEventMotion *event,
 		return FALSE;
 	}
 	
-	if ((notebook->priv->drag_in_progress == FALSE) 
-	    && (gtk_drag_check_threshold (GTK_WIDGET (notebook),
-					  notebook->priv->x_start, 
-					  notebook->priv->y_start, 
-					  event->x_root, event->y_root)))
+	if (notebook->priv->drag_in_progress == FALSE)
 	{
-		gint cur_page;
-		
-		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-		drag_start (notebook, notebook, cur_page); 
+		if (gtk_drag_check_threshold (GTK_WIDGET (notebook),
+					      notebook->priv->x_start, 
+					      notebook->priv->y_start, 
+					      event->x_root, event->y_root))
+		{
+			drag_start (notebook);
+		}
+		else
+		{
+			return FALSE;
+		}
 	}
 
 	result = find_notebook_and_tab_at_pos ((gint)event->x_root, 
@@ -561,13 +600,27 @@ button_press_cb (GulNotebook *notebook,
 					  G_CALLBACK (motion_notify_cb), NULL);
 	}
 
+	/* Switch tabs on right click. This means that when the popup
+	 * menu is shown, it will be clear that it acts on the tab under
+	 * the mouse. popup menu is attached by the GulNotebook user if
+	 * the default is not desired.
+	 */
+	else if ((event->button == 3) && (event->type == GDK_BUTTON_PRESS))
+	{
+		if (tab_clicked != -1)
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook),
+						      tab_clicked);
+		else
+			return TRUE;
+	}
+
 	return FALSE;
 }
 
 GtkWidget *
 gul_notebook_new (void)
 {
-	return GTK_WIDGET (g_object_new (GUL_NOTEBOOK_TYPE, NULL));
+	return GTK_WIDGET (g_object_new (GUL_TYPE_NOTEBOOK, NULL));
 }
 
 static void
@@ -581,12 +634,20 @@ gul_notebook_switch_page_cb (GtkNotebook *notebook,
 	
 	child = gtk_notebook_get_nth_page (notebook, page_num);
 
-	/* Remove the old page, we dont want to grow unnecessarily
-	 * the list */
-	if (nb->priv->focused_pages)
+	if (nb->priv->automatic_tab_switch)
 	{
-		nb->priv->focused_pages = 
-			g_list_remove (nb->priv->focused_pages, child);
+		/* Remove the old page, we dont want to grow unnecessarily
+		 * the list */
+		if (nb->priv->focused_pages)
+		{
+			nb->priv->focused_pages = 
+				g_list_remove (nb->priv->focused_pages, child);
+		}
+	}
+	else
+	{
+		g_list_free (nb->priv->focused_pages);
+		nb->priv->focused_pages = NULL;
 	}
 
 	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
@@ -600,15 +661,24 @@ gul_notebook_switch_page_cb (GtkNotebook *notebook,
 static void
 gul_notebook_init (GulNotebook *notebook)
 {
+	if (!_gul_notebook_tooltips)
+	{
+		_gul_notebook_tooltips = gtk_tooltips_new ();
+		g_object_add_weak_pointer (G_OBJECT (_gul_notebook_tooltips),
+					   (gpointer)&_gul_notebook_tooltips);
+	}
+	g_object_ref (_gul_notebook_tooltips);
+	gtk_object_sink (GTK_OBJECT (_gul_notebook_tooltips));
+
 	notebook->priv = g_new (GulNotebookPrivate, 1);
 
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
 	notebook->priv->focused_pages = NULL;
+	notebook->priv->automatic_tab_switch = FALSE;
 	notebook->priv->opened_tabs = NULL;
 	notebook->priv->policy = GTK_POLICY_AUTOMATIC;
+	notebook->priv->tab_width_chars = DEFAULT_TAB_WIDTH_CHARS;
 	
 	notebooks = g_list_append (notebooks, notebook);
 
@@ -634,6 +704,10 @@ gul_notebook_finalize (GObject *object)
 	g_list_free (notebook->priv->opened_tabs);
 
 	g_free (notebook->priv);
+
+	g_object_unref (_gul_notebook_tooltips);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -641,39 +715,74 @@ tab_close_button_clicked_cb (GtkWidget *widget,
 			     GtkWidget *child)
 {
 	GulNotebook *notebook;
+	gboolean inhibited = FALSE;
 
 	notebook = GUL_NOTEBOOK (gtk_widget_get_parent (child));
-	gul_notebook_remove_page (notebook, child);
+
+	g_signal_emit (G_OBJECT (notebook), gul_notebook_signals[TAB_DELETE],
+		       0, child, &inhibited);
+
+	if (inhibited == FALSE)
+	{
+		gul_notebook_remove_page (notebook, child);
+	}
 }
 
 static void
-tab_close_button_state_changed_cb (GtkWidget	*widget,
-		                   GtkStateType	 previous_state)
+tab_label_style_set (GtkWidget *label, GtkStyle *previous_style, GtkWidget *hbox)
 {
-	if (GTK_WIDGET_STATE(widget) == GTK_STATE_ACTIVE)
-	{
-		gtk_widget_set_state (widget, GTK_STATE_NORMAL);
-	}
+	GulNotebook      *notebook;
+	PangoFontMetrics *metrics;
+	PangoContext	 *context;
+	gint	          char_width;
+	gint              n_pixels;
+
+	notebook = GUL_NOTEBOOK (gtk_widget_get_ancestor (hbox, GUL_TYPE_NOTEBOOK));
+
+	context = gtk_widget_get_pango_context (label);
+	metrics = pango_context_get_metrics (context,
+			                     label->style->font_desc,
+					     pango_context_get_language (context));
+
+	char_width = pango_font_metrics_get_approximate_char_width (metrics);
+	pango_font_metrics_unref (metrics);
+
+	n_pixels = notebook->priv->tab_width_chars * PANGO_PIXELS(char_width);
+
+	/* We need to set the width of the widget that contains not only the
+	 * label, but also the favicon as otherwise tabs with icon would be
+	 * wider and we want to avoid that.  Consequently we need to take the
+	 * icon into account so that we really have the space for 'n'
+	 * characters when the icon is shown (and a little more when not.)
+	 */
+
+	n_pixels += 16;
+	n_pixels += SPACING;
+
+	hbox = g_object_get_data (G_OBJECT (hbox), "label-hbox");
+	gtk_widget_set_size_request (hbox, n_pixels, -1);
 }
 
 static GtkWidget *
 tab_build_label (GulNotebook *nb, GtkWidget *child)
 {
-	GtkWidget *label, *hbox, *close_button, *image;
+	GtkWidget *label, *hbox, *close_button, *image, *label_hbox;
 	GtkRequisition size;
 	GtkRcStyle *rcstyle;
-	GClosure *closure;
 	GtkWidget *window;
+	GtkWidget *label_ebox;
+	GtkWidget *alignment;
 
 	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
 	
-	/* set hbox spacing and label padding (see below) so that there's an
-	 * equal amount of space around the label */
-	hbox = gtk_hbox_new (FALSE, 4);
+	/* Add some space between the label and the close button, this
+	 * should be the same as the left-padding for the label */
+	hbox = gtk_hbox_new (FALSE, SPACING);
 
 	/* setup close button, zero out {x,y}thickness to get smallest possible
 	 * size */
 	close_button = gtk_button_new ();
+	gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (close_button), 
 			       GTK_RELIEF_NONE);
 	rcstyle = gtk_rc_style_new ();
@@ -687,23 +796,28 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
 	gtk_widget_set_size_request (close_button, size.width, size.height);
 	gtk_container_add (GTK_CONTAINER (close_button), image);
 
-	/* setup label */ 
+	gtk_tooltips_set_tip (_gul_notebook_tooltips, close_button,
+			      _("Close this tab"), NULL);
+
+	/* setup label, need eventbox for tooltips and left-padding to get
+	 * comfortable space between the label and icon/tab border */
+	label_ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
+	label_hbox = gtk_hbox_new (FALSE, 0);
         label = gtk_label_new (_("Untitled"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
-        gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 
-	tab_label_set_size (GTK_WIDGET (window), hbox);
-		
-	closure = g_cclosure_new (G_CALLBACK (tab_label_size_request_cb), 
-				  child, NULL);
-  	g_object_watch_closure (G_OBJECT (label), closure);
-  	g_signal_connect_closure_by_id (G_OBJECT (window),
-		g_signal_lookup ("size_request", 
-				 G_OBJECT_TYPE (G_OBJECT (window))), 0,
-                                 closure,
-                                 FALSE);
-	
+	alignment = g_object_new (GTK_TYPE_ALIGNMENT,
+				  "xalign",       0.0,
+				  "left-padding", SPACING,
+				  NULL);
+	gtk_container_add (GTK_CONTAINER (alignment), label);
+	gtk_widget_show (alignment);
+	gtk_box_pack_end (GTK_BOX (label_hbox), alignment, TRUE, TRUE, 0);
+
+	gtk_box_pack_start (GTK_BOX (hbox), label_ebox, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (label_ebox), label_hbox);
+
 	/* setup button */
 	gtk_box_pack_start (GTK_BOX (hbox), close_button,
                             FALSE, FALSE, 0);
@@ -712,20 +826,20 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
                           G_CALLBACK (tab_close_button_clicked_cb),
                           child);
 
-	/* work around a cosmetic bug in gtk where buttons with GTK_RELIEF_NONE
-	 * have annoyingly visible border only when in GTK_STATE_ACTIVE (ie. in
-	 * inactive tabs) */
-	g_signal_connect (G_OBJECT (close_button), "state_changed",
-                          G_CALLBACK (tab_close_button_state_changed_cb),
-                          child);
-		
 	gtk_widget_show (hbox);
+	gtk_widget_show (label_ebox);
+	gtk_widget_show (label_hbox);
 	gtk_widget_show (label);
 	gtk_widget_show (image);
 	gtk_widget_show (close_button);
 
 	g_object_set_data (G_OBJECT (hbox), "label", label);
-	
+	g_object_set_data (G_OBJECT (hbox), "label-ebox", label_ebox);
+	g_object_set_data (G_OBJECT (hbox), "label-hbox", label_hbox);
+	g_object_set_data (G_OBJECT (hbox), "icon", 0);
+
+	g_signal_connect (label, "style-set", G_CALLBACK (tab_label_style_set), hbox);
+
 	return hbox;
 }
 
@@ -794,23 +908,23 @@ gul_notebook_insert_page (GulNotebook *nb,
 
 	if (jump_to)
 	{
+		nb->priv->automatic_tab_switch = TRUE;
+
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
 					       position);
-		g_object_set_data (G_OBJECT (child), "jump_to", 
-				   GINT_TO_POINTER (jump_to));
+
+		nb->priv->automatic_tab_switch = FALSE;
 	}
+
+	g_signal_emit (G_OBJECT (nb), gul_notebook_signals[TAB_ADDED], 
+		       0, child);
 }
 
 static void
 smart_tab_switching_on_closure (GulNotebook *nb,
 			  	GtkWidget *child)
 {
-	gboolean jump_to;
-	
-	jump_to = GPOINTER_TO_INT (g_object_get_data 
-				   (G_OBJECT (child), "jump_to"));
-
-	if (!jump_to || !nb->priv->focused_pages)
+	if (!nb->priv->focused_pages)
 	{
 		gtk_notebook_next_page (GTK_NOTEBOOK (nb));
 	}
@@ -825,8 +939,13 @@ smart_tab_switching_on_closure (GulNotebook *nb,
 		child = GTK_WIDGET (l->data);
 		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 						  child);
+
+		nb->priv->automatic_tab_switch = TRUE;
+		
 		gtk_notebook_set_current_page 
 			(GTK_NOTEBOOK (nb), page_num);
+
+		nb->priv->automatic_tab_switch = FALSE;
 	}
 }
 
@@ -836,6 +955,7 @@ gul_notebook_remove_page (GulNotebook *nb,
 {
 	int position, cur;
 	gboolean last_tab;
+	GtkWidget *label, *ebox;
 
 	last_tab = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb)) == 1;
 	if (last_tab)
@@ -862,9 +982,30 @@ gul_notebook_remove_page (GulNotebook *nb,
 		smart_tab_switching_on_closure (nb, child);
 	}
 	
+ 	/* remove the tool tip for this ebox */
+ 	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+ 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
+ 	gtk_tooltips_set_tip (_gul_notebook_tooltips, ebox, NULL, NULL);
+ 
+	/* If we don't clear the icon then for some reason it gets reset back
+	 * to an empty icon. This means that although gul_notebook_move_page() has
+	 * a ref to the icon, when it is placed in the new notebook, it is empty */
+	gul_notebook_set_page_icon (nb, child, 0);
+
+	/**
+	 * we ref the child so that it's still alive while the tabs_removed
+	 * signal is processed.
+	 */
+	g_object_ref (child);
+ 
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
 
 	update_tabs_visibility (nb, FALSE);
+ 
+	g_signal_emit (G_OBJECT (nb), gul_notebook_signals[TAB_REMOVED], 
+		       0, child);
+ 
+	g_object_unref (child);
 }
 
 void
@@ -888,11 +1029,48 @@ gul_notebook_set_page_title (GulNotebook *nb,
 			     GtkWidget *child,
 			     const char *title)
 {
-	GtkWidget *label;
-	
-	label = tab_get_label (nb, child);
+	GtkWidget *label, *ebox, *tab_label;
+	gchar *short_title;
+
+	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+
+	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label-ebox"));
+	label = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label"));
+	short_title = gul_string_shorten (title, MENU_ITEM_MAX_LENGTH);
+
 	gtk_label_set_label (GTK_LABEL (label), title);
-	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (nb), child, title);
+	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (nb), child, short_title);
+	gtk_tooltips_set_tip (_gul_notebook_tooltips, ebox, title, NULL);
+
+	g_free (short_title);
+}
+
+
+void
+gul_notebook_set_page_icon (GulNotebook *nb,
+			    GtkWidget *child,
+			    GtkWidget *icon)
+{
+	GtkWidget *old_icon;
+	GtkWidget *label, *hbox;
+
+	label    = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+	hbox     = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-hbox"));
+	old_icon = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "icon"));
+
+	if (icon == old_icon) return;
+
+	if (old_icon)
+	{
+		gtk_container_remove (GTK_CONTAINER (hbox), old_icon);
+	}
+
+	if (icon)
+	{
+		gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+	}
+
+	g_object_set_data (G_OBJECT (label), "icon", icon);
 }
 
 void
